/* DagHasLoopException.java */


package kb7.structures;


/**
 * Thrown if a loop is detected in a Dag.
 */
public final class DagHasLoopException extends Exception
{
  /**
   * This is the vertex where the loop was detected.
   */
  public Object currentVertex;


  private boolean specialConstructor;
  // This variable determines which constructor was used to create
  // this object and thereby affects the semantics of the
  // "getMessage" method (see below).


  /**
    * This constructor is used by "sort" in class Dag.
    * Calling this constructor generates
    * a new DagHasLoopException with the field "vertex" set.,
    * "expectedTokenSequences", and "tokenImage" set.  The boolean
    * flag "specialConstructor" is also set to true to indicate that
    * this constructor was used to create this object.
    * This constructor calls its super class with the empty string
    * to force the "toString" method of parent class "Throwable" to
    * print the error message in the form:
    *   DagHasLoopException: <result of getMessage>
    */
  public DagHasLoopException( Object v )
  {
    super("");
    this.specialConstructor = true;
    this.currentVertex = v;
  }


  /**
   * The following constructors are for use by you for whatever
   * purpose you can think of.  Constructing the exception in this
   * manner makes the exception behave in the normal way - i.e., as
   * documented in the class "Throwable".  The field "vertex"
   * does not contain relevant information.
   */
  public DagHasLoopException( String message )
  {
    super( message );
  }

  public DagHasLoopException()
  {
  }


  /**
   * This method has the standard behavior when this object has been
   * created using the standard constructors.  Otherwise, it uses
   * "currentVertex" to generate an error message and returns that.
   */
  public String getMessage()
  {
    if ( ! this.specialConstructor )
      return super.getMessage();

    return "Vertex = " + this.currentVertex;
  }
}
