/* EquivalenceClasses.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Hashtable;
import java.lang.StringBuffer;


/**
This class implements a set of equivalence classes.
*/


public class EquivalenceClasses
{
  private Map domain_to_classes = new Map();
  private Set classes = new Set();


  public EquivalenceClasses()
  {
  }


  public synchronized void addElement( Object o )
  {
    if ( !this.domain_to_classes.contains(o) ) {
      Set o_class = new Set();
      o_class.addElement( o );
      this.classes.addElement( o_class );
      domain_to_classes.put( o, o_class );
    }
  }

  public synchronized void put( Object o )
  {
    addElement( o );
  }


  public void equate( Object o1, Object o2 )
  {
    /* We implement this by uniting [o1] and [o2],
       replacing [o1] by the result, and then remapping
       all members of [o2] so they point at [o1].
       If [o2] is larger than [o1], we reverse the order
       of the arguments, so that we are always moving
       the smaller set.
    */
    if ( ((Set)this.domain_to_classes.get(o1)).size() >
         ((Set)this.domain_to_classes.get(o2)).size() )
      equate_1( o1, o2 );
    else
      equate_1( o2, o1 );
  }

  public void equate( Pair p )
  {
    equate( p.getFirst(), p.getSecond() );
  }

  /* Called by equate. [o2] is the set that's deleted, so
     for efficiency, it must be the smaller one.
  */
  private void equate_1( Object o1, Object o2 )
  {
    Set c1 = (Set)this.domain_to_classes.get(o1);
    Set c2 = (Set)this.domain_to_classes.get(o2);
    c1.unite( c2 );
    this.classes.deleteElement( c2 );
    Enumeration c2_elements = c2.elements();
    while ( c2_elements.hasMoreElements() ) {
      Object c2_x = c2_elements.nextElement();
      this.domain_to_classes.put( c2_x, c1 );
    }
  }


  public void unite( Set s2 )
  {
    unite( s2.elements() );
  }

  public void unite( Map m2 )
  {
    unite( m2.domainElements() );
  }

  public void unite( Enumeration s2_elements )
  {
    while ( s2_elements.hasMoreElements() ) {
      Object s2_x = s2_elements.nextElement();
      addElement( s2_x );
    }
  }


  public Set codomain()
  {
    return this.classes;
  }


  public Enumeration codomainElements()
  {
    return this.classes.elements();
  }


  public int domainSize()
  {
    return this.domain_to_classes.size();
  }

  public int codomainSize()
  {
    return this.classes.size();
  }


  public String toString()
  {
    return this.domain_to_classes.toString() + "/" +
           this.classes.toString();
  }
}
