/* IntegerMap.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.StringBuffer;


/**
 * This class implements a mapping from objects to ints. Since
 * these are not objects, and can only be stored in tables by
 * wrapping them in an Integer, it adds some convenience methods
 * for putting and getting them directly.
 */
public class IntegerMap extends Map
{  
  private int dflt = -1;
  // The lookup methods can't return null if the element
  // they're looking up has no image, so they return this
  // instead.


  /** 
   * Do-nothing constructor.
   */  
  public IntegerMap()
  {
    super();
  }


  /**
   * Builds an IntegerMap whose elements are given by -table-.
   * Does not check that the image elements are integers!
   */ 
  public IntegerMap( Hashtable table )
  {
    super( table );
  }


  /**
   * Builds an IntegerMap whose elements are given by -pairs-, where
   * each pair's second element is the image of the first.
   * Does not check that the image elements are integers!
   */ 
  public IntegerMap( Vector pairs )
  {
    super( pairs );
  }


  /**
   * Sets the default.
   */
  public final void setDefault( int dflt )
  {
    this.dflt = dflt;
  }


  /**
   * New method to add an int image directly.
   */
  public synchronized void put( Object o1, int i2 )
  {
    put( o1, new Integer(i2) );
  }

  /**
   * New method to add an int image directly.
   */
  public synchronized void put( int i1, int i2 )
  {
    put( i1, new Integer(i2) );
  }


  /**
   * New method to return an int image directly.
   */
  public synchronized int getint( Object key )
  {
    Integer i = (Integer)get(key);
    if ( i==null )
      return this.dflt;
    else
      return i.intValue();
  } 

  /**
   * New method to return an int image directly.
   */
  public synchronized int getint( int key )
  {
    return getint( new Integer(key) );
  }
}
