/* IntegerSet.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Vector;
import java.lang.StringBuffer;
import kb7.general.Bug;


/**
 * This class implements a set of integers.
 */
public final class IntegerSet 
             implements Cloneable,
                        java.io.Serializable
{
  private static final long serialVersionUID = 1000;


  private Vector table = new Vector();


  public IntegerSet()
  {
  }

  public IntegerSet( Vector table )
  {
    this.table = table;
  }

  public IntegerSet( int i )
  {
    this();
    addElement(i);
  }


  public final synchronized void addElement( int i )
  {
    if ( !this.contains(i) )
      this.table.addElement( new Integer(i) );
  }

  public final synchronized void put( int i )
  {
    addElement(i);
  }


  public final synchronized void deleteElement( int i )
  {
    for ( int n=0; n<this.table.size(); n++ )
      if ( ((Integer)this.table.elementAt(n)).intValue() == i ) {
        this.table.removeElementAt(n);
        return;
      }
  }

  public final synchronized void remove( int i )
  {
    deleteElement( i );
  }


  public final void unite( IntegerSet s2 )
  {
    unite( s2.elements() );
  }

  public final void unite( Enumeration s2_elements )
  {
    while ( s2_elements.hasMoreElements() ) {
      Integer s2_x = (Integer)s2_elements.nextElement();
      addElement( s2_x.intValue() );
    }
  }

  public final void unite( Vector s2 )
  {
    unite( s2.elements() );
  }


  public final Set union( Set s2 )
  {
    Set result = (Set)this.clone();
    result.unite( s2 );
    return result;
  }


  public final int size()
  {
    return this.table.size();
  }


  public final boolean isEmpty()
  {
    return this.table.size() == 0;
  }


  public final Enumeration elements()
  {
    return this.table.elements();
  }


  public final boolean contains( int i )
  {
    for ( int n=0; n<this.table.size(); n++ )
      if ( ((Integer)this.table.elementAt(n)).intValue() == i )
        return true;
    return false;
  }


  /*
   * Assumes the set to be a singleton, and returns the only
   * element. 
   */
  public final int theElement()
  {
    if ( size() == 0 ) 
      throw new Bug( "Set has no elements" );
    else if ( size() > 1 )
      throw new Bug( "Set has more than one element" );
    else 
      return ( (Integer)this.table.elementAt(0) ).intValue();
  }


  /*
   * Returns one of the elements. There are a few occasions when this
   * is useful, even for non-singletons.
   */
  public final int chooseElement()
  {
    if ( size() == 0 ) 
      throw new Bug( "Set has no elements" );
    else 
      return ( (Integer)this.table.elementAt(0) ).intValue();
  }


 /**
   * Overrides the Object.clone method, by copying the table.
   */
  public final Object clone()
  {
    return new IntegerSet( (Vector)this.table.clone() );
  }


  public final String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append( "{" );
    for ( int n=0; n<this.table.size(); n++ ) {
      sb.append( " " );
      sb.append( ((Integer)this.table.elementAt(n)).intValue() );
      if ( n < this.table.size()-1 )
        sb.append( "," );
      if ( this.table.size() > 0 )
        sb.append( " " );
    }
    sb.append( "}" );
    return sb.toString();
  }
}
