/* MapStack.java */


package kb7.structures;

import java.util.Stack;


/**
 * This class implements a stack of maps. This behaves like the structures
 * used to keep track of scopes in compiler-writing. A new MapStack has
 * one map, to which put(key.value) and get(key) refer. But you can push a 
 * new map on the stack by calling openScope(). put(key,value) then affects 
 * this. get(key) searches the stack until it finds a map in which the image
 * of key is non-null, and returns that. If there is no such map, it returns
 * null.
 */
public class MapStack extends Function
{
  private Stack maps = new Stack();


  public static void main( String[] argv )
  {
    MapStack maps = new MapStack();
    maps.put( "1", "one" );
    maps.put( "2", "two" );
    System.out.println( maps.get("1") );    
    System.out.println( maps.get("2") );    
    System.out.println( maps.get("3") );
    System.out.println();

    maps.openScope();
    maps.put( "2", "twee" );
    maps.put( "3", "drie" );    
    System.out.println( maps.get("1") );    
    System.out.println( maps.get("2") );
    System.out.println( maps.get("3") );
    System.out.println();

    maps.openScope();
    maps.put( "3", "drei" );  
    System.out.println( maps.get("1") );    
    System.out.println( maps.get("2") );
    System.out.println( maps.get("3") );
    System.out.println();

    maps.closeScope();
    System.out.println( maps.get("1") );    
    System.out.println( maps.get("2") );
    System.out.println( maps.get("3") );
    System.out.println();

    maps.closeScope();
    System.out.println( maps.get("1") );    
    System.out.println( maps.get("2") );
    System.out.println( maps.get("3") );
    System.out.println();

    maps.closeScope();
    System.out.println( maps.get("1") );    
    System.out.println( maps.get("2") );
    System.out.println( maps.get("3") );
    System.out.println();
  }

  
  public MapStack()
  {
    maps.push( new Map() );
  }


  public final void openScope()
  {
    this.maps.push( new Map() );
  }

  public final void openScope( Map map )
  {
    this.maps.push( map );
  }

  public final void closeScope()
  {
    this.maps.pop();
  }


 /**
   * Adds a mapping from o1 to o2.
   */
  public final synchronized void put( Object o1, Object o2 )
  {
    ((Map)this.maps.peek()).put( o1, o2 );
  }


  /**
   * Adds a mapping from p.getFirst() to p.getSecond().
   */
  public final synchronized void addElement( Pair p )
  {
    ((Map)this.maps.peek()).put( p.getFirst(), p.getSecond() );
  }


  /**
   * Returns the image of key.
   */
  public final synchronized Object get( Object key )
  {
    for ( int i=this.maps.size()-1; i>=0; i-- ) {
      Object image = ((Map)this.maps.elementAt(i)).get( key );
      if ( image != null )
        return image;
    }
    return null;
  }


  public final String toString()
  {
    return "[MapStack " + this.maps + "]";
  }
}
