(module-name 'PageFileToFunction)

(module-static #t)


; Reads filename and returns the corresponding
; page-file function.
;
(define (page-file-to-function (filename <string>)
                               (document-root <string>)) <procedure>
  (eval (page-file-to-lambda-expression filename document-root))
)


; Reads filename and returns the corresponding
; page-file lambda-expression.
;
(define (page-file-to-lambda-expression (filename <string>)
                                        (document-root <string>)) <list>
  (let* ( (components (page-file-to-components filename document-root))
          (args (car components))
          (body (cadr components))
          (lambda-expr (make-page-lambda-expression args body document-root))
        )
    lambda-expr
  )
)


; Returns a two-element list <outputs, list>.
; The first element is the arguments from the page function header, 
; the second is the remaining S-expressions making up the function
; body.
;
(define (page-file-to-components (filename <string>)
                                 (document-root <string>)) <list>
  (let* ( (s-exprs (page-file-to-list filename document-root))
        )
    (if (eq? s-exprs '())
      (list '() '())
    ;else 
      (if (eq? (caar s-exprs) 'args)
        (list (cdar s-exprs) (cdr s-exprs))
      ;else
        (list '() s-exprs)
      )
    )
  )
)


; Returns a list of the S-expressions in filename,
; after replacing ]...[ sequences in the file
; by quoted strings. Throws WSM exceptions on file
; not found or syntax error.
;
(define (page-file-to-list (filename <string>)
                           (document-root <string>)) <list>
  (let* ( (contents #!null)
          (preprocessed-contents #!null)
          (preprocessed-contents-as-string #!null)
        )
;    (try-catch
      (begin
        (set! contents (invoke-static <kb7.wsm.Utilities> 'fileToString 
(invoke filename 'toString)
))
        (set! preprocessed-contents (invoke-static <kb7.wsm.PreProcessPageFile> 'preProcessPageFile contents))
        (set! preprocessed-contents-as-string (string-append preprocessed-contents))
        (call-with-input-string preprocessed-contents-as-string readall)
      )
;    ;exceptionally
;      (ex <java.io.FileNotFoundException>
;        (primitive-throw (make <kb7.wsm.PageNotFoundException> filename))
;      )
;      (ex <gnu.text.SyntaxException>
;        (primitive-throw (make <kb7.wsm.PageSyntaxException> 
;                               (invoke (invoke ex 'getMessages)
;'toString)
;                         )
;        )
;      )
;      (ex <java.lang.RuntimeException>
;        (primitive-throw (make <kb7.wsm.OtherException> 
;                               (invoke ex 'getMessage)
;                         )
;        )
;      )
;    )
  )
)


; Returns a list of the S-expressions
; left on port p.
;
(define (readall (p <input-port>)) <list>
  (let ( (item (read p))
       )
    (if (eq? item #!eof)
      '()
    ;else
      (cons item (readall p))
    )
  )
)


; Takes the list of S-expressions from
; a page file and constructs
; the relevant function.
;
(define (make-page-lambda-expression (args <list>)
                                     (exprs <list>)
                                     (document-root <string>)) <list>
  (let* ( (body-display-part 
            (map (lambda (expr)
                   (cond
                     ((and (list? expr)
                           (eq? (car expr) 'define)
                      )
                        expr
                        ; If expr is a (define ...).
                     )
                     ((and (list? expr)
                           (string? (car expr))
                      )
                        (let* ( (page-name (car expr))
                                (page-and-dir (invoke-static <kb7.wsm.Utilities> 'joinDirToFile 
(invoke document-root 'toString) 
(invoke page-name 'toString)
                                              )
                                )
                                (full-page-name (string-append
(invoke-static <kb7.wsm.Utilities> 'makeFullFilename page-and-dir 
(invoke "" 'toString) 
'.pfn)))
                              )
                          `(set! RESULT (string-append RESULT (call-page ,full-page-name (list ,@(cdr expr)) ,document-root)))
                           ; If expr is a list starting with a string,
                           ; i.e. calls another page file.
                        )
                     )
                     (else
                        `(set! RESULT (string-append RESULT ,expr))
                        ; If expr is a normal expression to be
                        ; displayed.
                     )
                   )
                 )
                 exprs
            )
          )
          (body `((let ( (RESULT "") ) ,@body-display-part RESULT)) )
        )
    
      `(lambda ,args <string> ,@body)
  )
  ; Expressions that are 'define's should be left alone, since
  ; they probably shouldn't be expected to have a value.
  ; See http://www.schemers.org/Documents/Standards/R5RS/HTML
  ; section 5.2.
)


; Converts the file at filename into a page function, and
; calls it on args, returning the result as a string.
;
(define (call-page (filename <string>)
                   (args <list>)
                   (document-root <string>)) <string>
  (let ( (fn (page-file-to-function filename document-root))
         )
    (apply fn args)
  )
)


