/* Table.java */


package kb7.html;

import kb7.general.Bug;


/**
 * This class enables tables to be accessed as two-dimensional
 * arrays of cells.
 */
public class Table
{
  public final static void main( String argv[] )
  {
    System.out.println( "<P>" );
    Table t = new Table( "BORDER", 1,1 );
    System.out.println( t.toString() );
    System.out.println( "</P>" );

    System.out.println( "<P>" );
    t = new Table( "BORDER", 3, 2 );
    System.out.println( t.toString() );
    System.out.println( "</P>" );

    System.out.println( "<P>" );
    t = new Table( "BORDER", 5, 4 );
    t.setCell( "1/1", 1,1 );
    t.setCell( "2/1", 2,1 );
    t.setCell( new Cell("3/1"), 3,1 );
    t.setCell( new Cell("BGCOLOR=red","4/1"), 4,1 );
    t.setCell( "BGCOLOR=green","5/1", 5,1 );
    t.setCellColspan( "BGCOLOR=blue", "colspan", 1,2, 5 );
    t.setCell( "1/3", 1,3 );
    t.setCellRowspan( "BGCOLOR=yellow", "rowspan", 2,3, 2 );
    t.setCell( "3/3", 3,3 );
    t.setCell( "4/3", 4,3 );
    t.setCell( "5/3", 5,3 );
    t.setCell( "1/4", 1,4 ); 
    t.setCellColspan( "BGCOLOR=orange", "colspan", 3,4, 3 );
    System.out.println( t.toString() );
    System.out.println( "</P>" );
    System.out.println( "<P>" );

    System.out.println( "<P>" );
    t = new Table( "BORDER", 4, 2 );
    t.setCell( "1/1", 1,1 );
    t.setCellColAndRowspan( "BGCOLOR=red", "col and rowspan", 2,1, 2, 2 );
    System.out.println( t.toString() );
    System.out.println( "</P>" );

    t = new Table( "BORDER", 2,2 );
    t.setCell( "1/1", 1,1 );
    t.setCell( "2/1", 2,1 );
    t.setCell( "1/2", 1,2 );
    t.setCell( "2/2", 2,2 );    
    t.insertRowAfter(1);
    t.insertColumnAfter(2);
    System.out.println( t.toString() );
    System.out.println( "</P>" );
  }


  private final static String eol = System.getProperty( "line.separator", "\n" );


  private String attributes = "";
  // HTML attributes for the entire table.

  private int nrows=1;
  private int ncols=1;
  // Number of rows and columns.

  private Cell[][] cells = { { new Cell() } };
  // The cells making up the table. Indexed as cells[row][col],
  // where both are based at 0. The 0-basing requires care when implementing
  // the cell set and get methods.
  // Most of the time, each element will be a Cell. However, if a cell in a row
  // or column before has a rowspan or colspan, then the element may have been 
  // set to null.
  
  private String[] row_attributes = { "" };
  // HTML attributes for each tr.


  public Table()
  { }

  public Table( int ncols, int nrows )
  {
    this.cells = new Cell[nrows][ncols];
    this.ncols = ncols;
    this.nrows = nrows;
    for ( int row=0; row<nrows; row++ )
      for ( int col=0; col<ncols; col++ )
        this.cells[row][col] = new Cell();
    this.row_attributes = new String[nrows];
    for ( int row=0; row<nrows; row++ )
        this.row_attributes[row] = "";
  }

  public Table( String attributes, int ncols, int nrows )
  {
    this( ncols, nrows );
    this.attributes = attributes;
  }


  public final void insertColumnAfter( int after_col )
  {
    Cell[][] old_cells = this.cells;
    this.cells = new Cell[this.nrows][this.ncols+1];
    for ( int row=0; row<this.nrows; row++ ) {
      for ( int col=0; col<after_col; col++ )
        this.cells[row][col] = old_cells[row][col];
      this.cells[row][after_col] = new Cell();
      for ( int col=after_col; col<this.ncols; col++ )
        this.cells[row][col+1] = old_cells[row][col];
    }
    this.ncols++;
  }


  public final void insertRowAfter( int after_row )
  {
    Cell[][] old_cells = this.cells;
    this.cells = new Cell[this.nrows+1][0];
    for ( int row=0; row<after_row; row++ )
      this.cells[row] = old_cells[row];
    this.cells[after_row] = new Cell[this.ncols];
    for ( int col=0; col<this.ncols; col++ )
      this.cells[after_row][col] = new Cell();
    for ( int row=after_row; row<this.nrows; row++ )
      this.cells[row+1] = old_cells[row];
    String[] old_row_attributes = this.row_attributes;
    this.row_attributes = new String[this.nrows+1];
    for ( int row=0; row<after_row; row++ )
      this.row_attributes[row] = old_row_attributes[row];
    this.row_attributes[after_row] = "";
    for ( int row=after_row; row<this.nrows; row++ )
      this.row_attributes[row+1] = old_row_attributes[row];
    this.nrows++;
  }


  public final void setRowAttributes( String attributes, int row )
  {
    this.row_attributes[row-1] = attributes;
  }


  public final void setCell( Cell cell, int col, int row )
  {
    checkCell( col, row );
    this.cells[row-1][col-1] = cell;
  }

  public final void setCell( String contents, int col, int row )
  {  
    setCell( new Cell(contents), col, row );
  }

  public final void setCell( String attributes, String contents, int col, int row )
  {  
    setCell( new Cell(attributes,contents), col, row );
  }


  public final void setCellColspan( Cell cell, int col, int row, int colspan )
  {  
    checkSpan( col+1, row, colspan-1, 1 );
    Cell copy = (Cell)cell.clone();
    copy.attributes += " COLSPAN="+colspan;
    // Set the array element to a copy of the cell with a different colspan attribute.
    setCell( copy, col, row ); 
    setRange( null, col+1, row, colspan-1, 1 );    
  }

  public final void setCellColspan( String contents, int col, int row, int colspan )
  {  
    checkSpan( col+1, row, colspan-1, 1 );
    setCell( new Cell("COLSPAN="+colspan,contents), col, row );
    setRange( null, col+1, row, colspan-1, 1 );    
  }

  public final void setCellColspan( String attributes, String contents, int col, int row, int colspan )
  {
    checkSpan( col+1, row, colspan-1, 1 );
    setCell( new Cell(attributes+" COLSPAN="+colspan,contents), col, row );
    setRange( null, col+1, row, colspan-1, 1 );
  }


  public final void setCellRowspan( Cell cell, int col, int row, int rowspan )
  {
    checkSpan( col, row+1, 1, rowspan-1 );
    Cell copy = (Cell)cell.clone();
    copy.attributes += " ROWSPAN="+rowspan;
    // Set the array element to a copy of the cell with a different rowspan attribute.
    setCell( copy, col, row ); 
    setRange( null, col, row+1, 1, rowspan-1 );    
  }

  public final void setCellRowspan( String contents, int col, int row, int rowspan )
  {
    checkSpan( col, row+1, 1, rowspan-1 );
    setCell( new Cell("ROWSPAN="+rowspan,contents), col, row ); 
    setRange( null, col, row+1, 1, rowspan-1 );
  }

  public final void setCellRowspan( String attributes, String contents, int col, int row, int rowspan )
  {
    checkSpan( col, row+1, 1, rowspan-1 );
    setCell( new Cell(attributes+" ROWSPAN="+rowspan,contents), col, row );
    setRange( null, col, row+1, 1, rowspan-1 );
  }


  public final void setCellColAndRowspan( Cell cell, int col, int row, int colspan, int rowspan )
  {
    checkSpan( col+1, row+1, colspan-1, rowspan-1 );
    Cell copy = (Cell)cell.clone();
    copy.attributes += " COLSPAN="+colspan+" ROWSPAN="+rowspan;
    // Set the array element to a copy of the cell with different colspan and rowspan attributes.
    setCell( copy, col, row ); 
    setRange( null, col+1, row+1, colspan-1, rowspan-1 );    
    setRange( null, col, row+1, 1, rowspan-1 );
    setRange( null, col+1, row, colspan-1, 1 );
  }

  public final void setCellColAndRowspan( String contents, int col, int row, int colspan, int rowspan )
  {
    checkSpan( col+1, row+1, colspan-1, rowspan-1 );
    setCell( new Cell("COLSPAN="+colspan+" ROWSPAN="+rowspan,contents), col, row ); 
    setRange( null, col+1, row+1, colspan-1, rowspan-1 );
    setRange( null, col, row+1, 1, rowspan-1 );
    setRange( null, col+1, row, colspan-1, 1 );
  }

  public final void setCellColAndRowspan( String attributes, String contents, int col, int row, int colspan, int rowspan )
  {
    checkSpan( col+1, row+1, colspan-1, rowspan-1 );
    setCell( new Cell(attributes+" COLSPAN="+colspan+" ROWSPAN="+rowspan,contents), col, row );
    setRange( null, col+1, row+1, colspan-1, rowspan-1 );
    setRange( null, col, row+1, 1, rowspan-1 );
    setRange( null, col+1, row, colspan-1, 1 );
  }


  /* Set a range of cells to 'contents'. In the 1-based indexing of the user's column and
     row indexing, the range is col..col+(length of column range)-1, row..(length of row range)-1.
     That is, all cells from col and row up to the length of the range.
  */
  private final void setRange( Cell contents, int col, int row, int col_range, int row_range )
  {
    for ( int row_offset=1; row_offset<=row_range; row_offset++ )   
      for ( int col_offset=1; col_offset<=col_range; col_offset++ )
        this.cells[ row+row_offset-2 ][ col+col_offset-2 ] = contents;  
  }


  public final int getWidth()
  {
    return this.ncols;
  }

  public final int getHeight()
  {
    return this.nrows;
  }


  private final void checkCell( int col, int row )
  {
    if ( col < 1 || col > this.ncols )
      throw new Bug( "Column out of range: " + col );
    if ( row < 1 || row > this.nrows )
      throw new Bug( "Row out of range: " + row );    
  }


  private final void checkSpan( int col, int row, int col_range, int row_range )
  {
    if ( col+col_range-1 > this.ncols )
      throw new Bug( "Column span out of range: " + col + " " + col_range );
    if ( row+row_range-1 > this.nrows )
      throw new Bug( "Row span out of range: " + row + " " + row_range );    
  }


  public final String toCSV( boolean strip_commas )
  {
    String result = "";
    for ( int row=0; row<this.nrows; row++ ) {
      String r = "";
      for ( int col=0; col<this.ncols; col++ ) {
         Cell cell = this.cells[row][col];
         String cell_string = ( cell==null ? "" : cell.toCSV() );
         if ( strip_commas ) 
           cell_string = kb7.strings.Strings.replace( cell_string, ",", " " );
         r += cell_string;
         if ( col <this.ncols-1 ) 
           r += " , ";
      }
      result += r + kb7.io.Files.eol;
    }
    return result;
  }  


  public final String toString()
  {
    String result = "";
    for ( int row=0; row<this.nrows; row++ ) {
      String r = "";
      for ( int col=0; col<this.ncols; col++ ) {
         Cell cell = this.cells[row][col];
         r += ( cell==null ? "" : cell.toString() );
      }
      result += Html.tr( this.row_attributes[row], r );
    }
    result = Html.table( this.attributes, result );
    return result;
  }  
}

