/* Http.java */


package kb7.http;

import java.util.Enumeration;
import java.util.Vector;
import javax.servlet.ServletRequest;
import kb7.general.Bug;
import kb7.strings.Strings;
import kb7.structures.Triple;


public class Http
{
  public static void main( String argv[] )
  {
    Vector v = new Vector();
    v.addElement( "new" );
    v.addElement( "submit_2" );
    v.addElement( "update" );
    String[] names = { "submit*", "cut" };
    System.out.println( findParameter( v.elements(), names ) ); 
    System.out.println( findParameters( v.elements(), names ) ); 

    v = new Vector();
    v.addElement( "new" );
    v.addElement( "submit_2" );
    v.addElement( "update" );
    String[] names_2 = { "update" };
    System.out.println( findParameter( v.elements(), names_2 ) ); 
    System.out.println( findParameters( v.elements(), names_2 ) ); 

    v = new Vector();
    v.addElement( "new" );
    v.addElement( "submit_2" );
    v.addElement( "update" );
    String[] names_3 = { "run", "print*" };
    System.out.println( findParameter( v.elements(), names_3 ) );
    System.out.println( findParameters( v.elements(), names_3 ) );     

    v = new Vector();
    v.addElement( "new" );
    v.addElement( "submit_2" );
    v.addElement( "update" );
    String[] names_4 = { "submit*", "update", "print*", "new" };
    System.out.println( findParameter( v.elements(), names_4 ) );
    System.out.println( findParameters( v.elements(), names_4 ) ); 
  }


  /**
   * Searches the query parameters for one that is either equal to 
   * a given name, or starts with a given prefix. 'patterns' contains
   * the patterns: if the final character is a *, then we search for
   * the prefix preceding it.
   * Returns a triple <full name, matched prefix, rest of name> .
   * <P>
   * This is useful when searching to see which of several SUBMIT
   * buttons was pressed.
   */
  public static Triple findParameter( ServletRequest req, String[] patterns )  
  {
    return findParameter( req.getParameterNames(), patterns );
  }
  
  /**
   * As above, but over an enumeration of names.
   */
  public static Triple findParameter( Enumeration names, String[] patterns )  
  {
    while ( names.hasMoreElements() ) {
      String name = (String)names.nextElement();
      for ( int i=0; i<patterns.length; i++ ) {
        Triple t = match( name, patterns );
        if ( t!= null )
          return t;
      }
    }
    return null;
  } 

  /**
   * As above, but returns a vector containing all matches. This will be
   * empty if there were none.
   */
  public static Vector findParameters( ServletRequest req, String[] patterns )  
  {
    return findParameters( req.getParameterNames(), patterns );
  }

  /**
   * As above, but over an Enumeration.
   */
  public static Vector findParameters( Enumeration names, String[] patterns )  
  {
    Vector matches = new Vector();
    while ( names.hasMoreElements() ) {
      String name = (String)names.nextElement();
      Triple t = match( name, patterns );
      if ( t != null )
        matches.addElement( t );
    }
    return matches;
  }


  /**
   * Given ...
   */
  public static final String[] getParameters( ServletRequest req, String value_prefix, String no_of_name )
  {
    String no_of_str = req.getParameter( no_of_name ).trim();
    int size = Integer.parseInt( no_of_str );
    String[] result = new String[ size ];
    for ( int i=0; i<size; i++ ) {
      String value_name = value_prefix + i;
      String value_str = req.getParameter( value_name ).trim();
      result[i] = value_str;
    }
    return result;
  }


  /* Matches a parameter name against an array of patterns, returning
     a triple < name, prefix, suffix > if it matches, otherwise null.
  */
  private static Triple match( String name, String[] patterns )
  {
    for ( int i=0; i<patterns.length; i++ ) {
      String suffix = Strings.matchesPrefix( name, patterns[i] );
      if ( suffix != null ) {
        String prefix = patterns[i];
        if ( prefix.endsWith("*") )
          prefix = prefix.substring( 0, prefix.length()-1 );
        return new Triple( name, prefix, suffix );
      }
    }        
    return null;
  }
}
