/* Files.java */


package kb7.io;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;


/**
 * This class defines various file-handling utilities.
 */
public final class Files
{
  /**
   * The end-of-line character.
   */
  public final static String eol = System.getProperty( "line.separator", "\n" );


  public static void main( String argv[] )
  {
    try {
      System.out.println( fileToString( "d:\\kb7\\io\\Files.java" ) );
      copy ( "d:\\kb7\\io\\Files.java", "d:\\kb7\\io\\test" );
    }
    catch ( Exception e ) {
      System.out.println( e );
    }
  }


  /**
   * Writes a string to a file. Does not add a newline at the end.
   */
  public final static void stringToFile( String filename, String s )
                           throws IOException
  {
    PrintWriter out = new PrintWriter( new FileWriter(filename) );
    out.print( s );
    out.close();
  }


  /**
   * Reads a file and returns the entire contents as a string.
   */
  public final static String fileToString( String filename )
                             throws IOException
  {
    int c;
    StringBuffer sb = new StringBuffer();
    FileReader in = new FileReader(filename);
    while ( ( c = in.read() ) != -1 ) {
      sb.append( (char)c );
    }
    in.close();
    return sb.toString();
  }


  /**
   * Copies the first file to the second. Does not add
   * a newline at the end.
   */
  public static final void copy( String from, String to ) 
                           throws IOException
  {
    FileReader in = new FileReader(from);
    FileWriter out = new FileWriter(to);
    int c;
    while ( ( c = in.read() ) != -1 ) 
      out.write( c );
    in.close();
    out.close();
  }


  /**
   * Copies all files named in the 'froms' vector to the 'to'
   * file, appending the second, third, ... to the first.
   * Does not add a newline at the end.
   */
  public static final void copy( Vector froms, String to )
                           throws IOException
  {
    PrintWriter out = new PrintWriter( new FileWriter(to) );
    for ( int i=0; i<froms.size(); i++ ) {
      FileReader in = new FileReader( (String)froms.elementAt(i) );
      int c;
      while ( ( c = in.read() ) != -1 ) 
        out.write( c );
      in.close();
    }
    out.close();
  }
}

