/* Files.java */


package kb7.Files;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;


/**
 * This class defines various file-handling utilities.
 */
public final class Files
{
  /**
   * The end-of-line character.
   */
  public final static String eol = System.getProperty( "line.separator", "\n" );


  /**
   * Reads a file and returns the entire contents as a string.
   */
  public final static String fileToString( String filename )
                             throws IOException
  {
    StringBuffer sb = new StringBuffer();
    String line;
    BufferedReader in = new BufferedReader( new FileReader(filename) );
    while ( ( line = in.readLine() ) != null ) {
      sb.append( line + eol );
    }
    in.close();
    return sb.toString();
  }

  
  /**
   * Copies the first file to the second.
   */
  public static final void copy( String from, String to ) 
                           throws IOException
  {
    BufferedReader in = new BufferedReader( new FileReader(from) );
    PrintWriter out = new PrintWriter( new FileWriter(to) );
    String line;
    while ( ( line = in.readLine() ) != null ) 
      out.println( line );
    in.close();
    out.close();
  }


  /**
   * Copies all files named in the 'froms' vector to the 'to'
   * file, appending the second, third, ... to the first.
   */
  public static final void copy( Vector froms, String to )
                           throws IOException
  {
    PrintWriter out = new PrintWriter( new FileWriter(to) );
    for ( int i=0; i<froms.size(); i++ ) {
      BufferedReader in = 
        new BufferedReader( new FileReader( (String)froms.elementAt(i) ) );
      String line;
      while ( ( line = in.readLine() ) != null ) 
        out.println( line );
      in.close();
    }
    out.close();
  }
}

