/* PatternElement.java */


package kb7.matcher;
import java.util.Vector;
import kb7.structures.Map;


/**
 * These classes provide a simple pattern matcher, with the ability to
 * match against constraints and literal values, and to assign to 
 * pattern variables. The main class is PatternElement, subclassed into
 * PatternVariable on the one hand, and PatternConstraint, further subclassed
 * into PatternValue, on the other. 
 * <P>
 * Each of these classes defines a method match( Object, Substitutions ).
 * A Substitutions is a map from PatternVariable to Object and records
 * variable bindings made during matching. Normally, one will match
 * a sequence of PatternElements against a sequence of objects, passing
 * the same Substitutions to each call of match. At the end of the match,
 * the Substitutions will contain all the variable bindings made. These may
 * be useful in computing some result of the match.
 * <P>
 * The class MatchState indicates the result of a match. It is subclassed
 * to MatchOK and MatchFailure, further subclassed to ConstraintFailure,
 * ValueFailure and BoundVariableFailure. ConstraintFailure and ValueFailure
 * indicate that a PatternConstraint or PatternVariable has failed to
 * match. BoundVariableFailure indicates that the value in a bound
 * variable has failed to match. These classes record the expected
 * constraint or value and the actual value, for use in error messages.
 */
public interface PatternElement
{
  public MatchState match( Object o, Substitutions s );

  public String toDisplay();
}



