/* PatternVariableImp.java */


package kb7.matcher;


/**
 * This class represents a variable. 
 */
public class PatternVariableImp implements PatternVariable
{
  String name;
  // The name of the variable. Used as a reference to it in
  // Substitutions.

  PatternElement constraint = null;
  // If set, acts as a constraint on the values the
  // variable can match.


  public PatternVariableImp( String name )
  { this.name = name; }

  public PatternVariableImp( String name, PatternElement constraint )
  { this(name);
    this.constraint = constraint;
  }


  public String getName()
  { return this.name; }

  public PatternElement getPatternElement( )
  { return this.constraint; }

  
  public void setName( String s )
  { this.name = s; }

  public void setPatternElement( PatternElement e )
  { this.constraint = e; }

  
  public final MatchState match( Object o, Substitutions s )
  {
    MatchState matched = null;

    PatternElement value = (PatternElement)s.get( this.name );
    if ( value == null ) {
      if ( this.constraint != null ) 
        matched = this.constraint.match( o, s );
      else 
        /* No constraint on variable. */
        matched = MatchState.MatchOK;
      if ( matched == MatchState.MatchOK ) 
        s.put( this.name, makePatternValue(o) );
      return matched;
    }
    else {
      /* Variable is already bound. */ 
      matched = value.match( o, s );
      if ( matched == MatchState.MatchOK )
        return MatchState.MatchOK;
      else      
        return new BoundVariableFailure(this.name,(ValueFailure)matched);
    }      
  }


  /**
   * When an unbound pattern variable is matched against
   * an object o, match creates an instance of PatternElement
   * to hold o, and assigns it to the map which maps
   * the variable's name to its value. Usually, that
   * instance of PatternElement will be a PatternValueImp.
   * But sometimes the match will not just be a simple
   * equality test, and may need other kinds of PatternElement.
   * This method can be overridden to provide one.
   */
  public PatternElement makePatternValue( Object o )
  {
    return new PatternValueImp( o ); 
  }
  

  public final String toDisplay()
  {
    return toString();
  }

  public final String toString()
  {
    return "[PatternVariableImp " + this.name + " " + this.constraint + " ]";
  }  
}


