/* Substitutions.java */

package kb7.matcher;

import kb7.structures.Map;


/**
 * This class represents a set of variable substitutions, i.e. a map
 * from pattern variables to values acquired during the match. Variables
 * are represented by their names, so two different PatternVariables with
 * the same name will be treated as the same variable. They are bound
 * to a PatternValue containing the value: this makes the matcher
 * simpler. We define special methods to put and get bindings.
 */
public class Substitutions extends Map
{
  public final void putBinding( String name, Object value )
  {
    put( name, new PatternValueImp(value) );
  }


  public final Object getBinding( String name )
  {
    if ( get(name) == null )
      return null;
    else
      return ( (PatternValueImp)get(name) ).getValue();
  }


 /** 
   * As toDisplay, but adds the class name.
   */
  public String toString()
  {
    return "[Substitutions " + toDisplay() + "]";
  }
}

