/* Persistence.java */


package kb7.persistence.Persistence;

import java.io.*;
import kb7.general.Bug;


/**
 * This class provides write and read methods for passing objects
 * to and from files. It uses Java's object serialization methods.
 * If either of the methods detects an exception, they just throw
 * it back as a Bug, since there's probably little we can do about it.
 */
public class Persistence
{
  /**
   * Writes o to filename.
   */
  public static void write( Serializable o, String filename )
  { 
    try {
      FileOutputStream ostream = new FileOutputStream(filename);
      ObjectOutputStream out = new ObjectOutputStream(ostream);
      out.writeObject(o);
      out.flush();
      ostream.close();
    }
    catch ( Throwable e ) {
      throw new Bug( "Error on writing object: " + o + " " + filename + " " + e );
    }
  }


  /**
   * Reads an object from filename.
   */
  public static Serializable read( String filename )
  { 
    try {
      FileInputStream istream = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(istream);
      Serializable s = (Serializable)in.readObject();
      istream.close();
      return s;
    }
    catch ( Throwable e ) {
      throw new Bug( "Error on reading object: " + filename + " " + e );
    }
  }
}
