/* Strings.java */


package kb7.strings;

import java.util.Vector;
import kb7.general.Bug;
import kb7.structures.Map;


/**
 * This class defines various useful string-handling methods.
 */
public final class Strings
{
  public static void main( String argv[] )
  {
    System.out.println( capitaliseFirst("a") );
    System.out.println( capitaliseFirst("abcde") );
    System.out.println( capitaliseFirst("Abcde") );
    System.out.println( capitaliseFirst("1bcde") );

    System.out.println( split("abc de fgh ij klmn opqrst",11) );
    System.out.println( split("abc de fgh ij klmn opqrst",10) );
    System.out.println( split("abc de fgh ij klmn opqrst",9) );
    System.out.println( split("abc de fgh ij klmn opqrst",6) ); 

    System.out.println( splitAt("","/") );
    System.out.println( splitAt("abcd","/") );
    System.out.println( splitAt("///","/") );
    System.out.println( splitAt("///","/\\") );
    System.out.println( splitAt("a12bc1def12ghij1klmno","12") );

    System.out.println( "'"+trimTrailing("")+"'" );
    System.out.println( "'"+trimTrailing("abc")+"'" );
    System.out.println( "'"+trimTrailing("   ")+"'" );
    System.out.println( "'"+trimTrailing("  abc  ")+"'" );

    System.out.println( "'"+removeWhitespace("")+"'" );
    System.out.println( "'"+removeWhitespace("abc")+"'" );
    System.out.println( "'"+removeWhitespace("   ")+"'" );
    System.out.println( "'"+removeWhitespace("  abc  ")+"'" );
    System.out.println( "'"+removeWhitespace("  a b c  ")+"'" );

    System.out.println( matchesPrefix("abcd","abcd") );     // ""
    System.out.println( matchesPrefix("abcd","abce") );     // null
    System.out.println( matchesPrefix("","abce") );         // null
    System.out.println( matchesPrefix("abcd","") );         // null
    System.out.println( matchesPrefix("abcd","ab*") );      // "cd"
    System.out.println( matchesPrefix("abcd","abc*") );     // "d" 
    System.out.println( matchesPrefix("abcd","abcd*") );    // "" 
    System.out.println( matchesPrefix("abcd","abcde*") );   // null
    System.out.println( matchesPrefix("xyz","y*") );        // null

    System.out.println( replace( "", "ab", "cd" ) );
    System.out.println( replace( "abcd", "ab", "xyz" ) );
    System.out.println( replace( "abcd", "cd", "xyz" ) );
    System.out.println( replace( "abc12def1ghi12jkl1mno2pqr", "12", "X" ) ); 
    System.out.println( replace( "abc12def1ghi12jkl1mno2pqr", "12", "" ) );

    System.out.println( take( "12_34", "_" ) );    
    System.out.println( take( "12<->34", "<->" ) );

    System.out.println( drop( "12_34", "_" ) ); 
    System.out.println( drop( "12<->34", "<->" ) );

    System.out.println( multiply( "abc", 3 ) );

    System.out.println( escape( "| |" ) );
    System.out.println( escape( "|\f\t\r\n\40|" ) );
    System.out.println( escape( "|\002\003|" ) );
    System.out.println( escape( "|a|" ) );

    System.out.println( unescape( "|\\040|" ) );
    System.out.println( unescape( "|\\40|" ) );
    System.out.println( unescape( "|\\fa\\tb\\rc\\nd\\40e|" ) );
    System.out.println( unescape( "|\\2\\3|" ) );
    System.out.println( unescape( "|\\02\\03|" ) );
    System.out.println( unescape( "|\\002\\003|" ) );
    System.out.println( unescape( "|\\0002\\0003|" ) );
    System.out.println( unescape( "|\\002123\\003456|" ) );
    System.out.println( unescape( "|\\101|" ) );
    System.out.println( unescape( "\\101" ) );

    Map replacements = new Map();
    replacements.put( "$one$", "1" );
    replacements.put( "$two$", "2" );
    replacements.put( "$three$", "3" );
    System.out.println( replaceInTemplate( "", replacements ) );
    System.out.println( replaceInTemplate( "$one$def$two$efg$three$", replacements ) );
    System.out.println( replaceInTemplate( "abc$one$def$two$efg$three$hij", replacements ) );
  }


  /**
   * Returns s with the first character capitalised, if a letter.
   */
  public static final String capitaliseFirst( String s )
  {
    String s1 = s.substring(0,1).toUpperCase() +
                s.substring(1);
    return s1;
  }


  /**
   * True if c is a letter.
   */
  public static final boolean isLetter( char c )
  {
    return (c>='a' && c<='z') || (c>='A' && c<='Z');
  }


  /**
   * Splits s into a sequence of substrings, each of length
   * max or less. It splits at spaces, and removes those spaces
   * from the result. The method is intended for reformatting
   * error messages and similar so that they do not extend
   * beyond the width of a line.
   */
  public static final Vector split( String s, int max )
  {
    Vector result = new Vector(); 
    s = s.trim();
    while ( s.length() > max ) {
      int i = max;
      while ( i>=0 && s.charAt(i)!=' ' )
        i--;
      while ( i>=0 && s.charAt(i)==' ' ) 
        i--;
      result.addElement( s.substring( 0, i+1 ) );  
      s = s.substring( i+2, s.length() );  
    } 
    result.addElement( s );
    return result;
  }


  /**
   * Splits s into a sequence of substrings at delim.
   * delim is not included in the resulting vector.
   */
  public static final Vector splitAt( String s, String delim )
  {
    Vector result = new Vector();
    while ( true ) {
      int index = s.indexOf( delim );
      if ( index == -1 ) {
        result.addElement( s );
        return result;
      }
      else {
        result.addElement( s.substring(0,index) );
        s = s.substring( index+delim.length() );
      }
    }
  }


  /**
   * Inverse of splitAt.
   */
  public static final String join( Vector v, String delim )
  {
    String result = "";
    for ( int i=0; i<v.size(); i++ ) {
      result += (String)v.elementAt(i);
      if ( i< v.size()-1 )
        result += delim;
    }
    return result;
  }


  /**
   * Counts the number of occurrences of delim in s.
   */
  public static final int count( String s, String delim )
  {
    int count=0;
    while ( true ) {
      int index = s.indexOf( delim );
      if ( index == -1 ) {
        return count;
      }
      else {
        count++;
        s = s.substring( index+delim.length() );
      }
    }
  }


  /** 
   * Removes all whitespace from s.
   */
  public static final String removeWhitespace( String s )
  {
    String result = "";
    for ( int i=0; i<s.length(); i++ ) {
      char c = s.charAt(i);
      if ( !Character.isWhitespace( c ) )
        result += c;
    }
    return result;
  }


  /**
   * Trims trailing whitespace from s.
   */
  public static final String trimTrailing( String s )
  {
    if ( s.length() == 0 )
      return s;
    else {
      char c = s.charAt( s.length()-1 );
      if ( Character.isWhitespace(c) )
        return trimTrailing( s.substring( 0, s.length()-1 ) );
      else
        return s;
    }
  }


  /**
   * If 'pattern' does not contain a star, returns null if the two strings
   * are unequal, "" otherwise.
   * If 'pattern' does contain a star, it must be the final character. Returns
   * null if 'name' does not start with the portion of 'pattern' before the star, and the
   * rest of 'name' otherwise.
   * This is useful when searching a list of strings for one that matches
   * one of a set of known prefixes.
   */
  public static String matchesPrefix( String s, String pattern )
  {
    if ( pattern.indexOf("*") == -1 ) {
      if ( pattern.equals(s) ) 
        return "";
      else
        return null;
    }
    else if ( pattern.indexOf("*") != pattern.length()-1 )
      throw new Bug( "* not at end of pattern: " + pattern );
    else {
      String pattern_pre = pattern.substring( 0, pattern.length()-1 );           
      if ( s.startsWith(pattern_pre) )
        return s.substring( pattern.length()-1, s.length() );
      else
        return null;
    }
  }


  /**
   * Replaces all occurrences of old in s by new.
   */
  public static String replace( String s, String old, String neww )
  {
    StringBuffer result = new StringBuffer();
    while ( true ) {
      int pos = s.indexOf( old );
      if ( pos == -1 ) {
        result.append( s );
        return result.toString();
      }
      else {
        String pre = s.substring( 0, pos );
        result.append( pre );
        result.append( neww );
        s = s.substring( pos+old.length(), s.length() ); 
      }
    }
  }


  /**
   * Returns the portion of s upto but not including the
   * first delimiter.
   */  
  public final static String take( String s, String delimiter ) 
  { 
    int delimiter_pos = s.indexOf(delimiter);
    if ( delimiter_pos == -1 )
      throw new Bug( "No delimiter in string: " + s );
    return s.substring( 0, delimiter_pos );
  }


  /**
   * Removes the portion of s upto and including the
   * first delimiter.
   */  
  public final static String drop( String s, String delimiter ) 
  { 
    int delimiter_pos = s.indexOf(delimiter);
    if ( delimiter_pos == -1 )
      throw new Bug( "No delimiter in string: " + s );
    return s.substring( delimiter_pos+delimiter.length() );
  }


  /**
   * Repeats s n times.
   */
  public final static String multiply( String s, int n )
  {
    String result = "";
    for ( int i=1; i<=n; i++ )
      result += s;
    return result;
  }


  /**
   * Used to convert raw characters to their escaped version
   * when these raw version cannot be used as part of an ASCII
   * string literal. Replaces spaces by \040. This means that
   * the result should appear as a contiguous sequence of
   * non-layout characters.
   */
  public final static String escape( String str ) 
  {
    /* Taken from code generated by JavaCC.
    */

    StringBuffer result = new StringBuffer();
    char ch;
    for ( int i = 0; i < str.length(); i++ ) {
      switch ( str.charAt(i) )
      {
        case 0 :
          continue;
        case ' ':
          result.append("\\040");
          continue;
        case '\b':
          result.append("\\b");
          continue;
        case '\t':
          result.append("\\t");
          continue;
        case '\n':
          result.append("\\n");
          continue;
        case '\f':
          result.append("\\f");
          continue;
        case '\r':
          result.append("\\r");
          continue;
        case '\"':
          result.append("\\\"");
          continue;
        case '\'':
          result.append("\\\'");
          continue;
        case '\\':
          result.append("\\\\");
          continue;
        default:
          if ( (ch = str.charAt(i)) < 0x20 || ch > 0x7e ) {
            String s = Integer.toString(ch, 8);
            result.append("\\" + s);
          } 
          else 
            result.append(ch);
          continue;
      }
    }
    return result.toString();
  }


  /**
   * Given a string containing escaped characters such as \n , 
   * converts them into the equivalent characters.
   */
  public final static String unescape( String s )
  {
    /* This code was modified from the Pnuts method parseString.
    */

    StringBuffer sb = new StringBuffer();
    char[] cc = s.toCharArray();
    boolean backquote = (cc[0] == '`');

    for ( int i=0; i<cc.length; i++ ) {
      if ( cc[i] == '\\' && !backquote ) {
        i++;
	switch ( cc[i] ) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
            int oct = cc[i] - '0';
            int oct_length = 1;
            while ( oct_length<3 && i<cc.length-1 && Character.isDigit( cc[i+1] ) ) {
              oct = oct*8 + (cc[i+1] - '0');
              i++;
              oct_length++;
            }
	    sb.append( (char)oct );
	    break;
          case 'b':
	    sb.append( '\b' );
	    break;
          case 't':
	    sb.append( '\t' );
            break;
          case 'n':
            sb.append( '\n' );
	    break;
          case 'f':
            sb.append( '\f' );	
            break;
          case 'r':
	    sb.append( '\r' );
	    break;
          case '"':
	    sb.append( '"' );
	    break;
          case '\\':
	    sb.append( '\\' );
	    break;
	}
      } 
      else 
        sb.append( cc[i] );
    }
    return sb.toString();
  }


  /**
   * Given a string containing substrings which begin and end with "$",
   * treats each of these as a key in replacements, and replaces each
   * by its image.
   */
  public final static String replaceInTemplate( String s, Map replacements )
  {
    String result = "";

    while ( true ) {
      int delimiter_pos = s.indexOf("$");
      if ( delimiter_pos == -1 ) 
        return result + s;
      else {
        String pre = take( s, "$" );
        result += pre;
        String post = drop( s, "$" );
        String key = take( post, "$" );
        s = drop( post, "$" );
        result += replacements.get( "$"+key+"$" );
      }
    }
  }
}
