/* RealMap.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.StringBuffer;


/**
 * This class implements a mapping from objects to doubles. Since
 * these are not objects, and can only be stored in tables by
 * wrapping them in a Double, it adds some convenience methods
 * for putting and getting them directly.
 */
public class RealMap extends Map
{
  public final static double NONE = -1;
  // The lookup methods can't return null if the element
  // they're looking up has no image, so they return this
  // instead.

  private double dflt = NONE;
  // The default value for none.

  
  /** 
   * Do-nothing constructor.
   */  
  public RealMap()
  {
    super();
  }


  /**
   * Builds an RealMap whose elements are given by -table-.
   * Does not check that the image elements are integers!
   */ 
  public RealMap( Hashtable table )
  {
    super( table );
  }


  /**
   * Builds an RealMap whose elements are given by -pairs-, where
   * each pair's second element is the image of the first.
   * Does not check that the image elements are integers!
   */ 
  public RealMap( Vector pairs )
  {
    super( pairs );
  }


  /**
   * Sets the default.
   */
  public final void setDefault( double dflt )
  {
    this.dflt = dflt;
  }


  /**
   * New method to add a double image directly.
   */
  public synchronized void put( Object o1, double d2 )
  {
    put( o1, new Double(d2) );
  }

  /**
   * New method to add a double image directly.
   */
  public synchronized void put( int i1, double d2 )
  {
    put( i1, new Double(d2) );
  }


  /**
   * New method to return a double image directly.
   */
  public synchronized double getdouble( Object key )
  {
    Double d = (Double)get(key);
    if ( d==null )
      return this.dflt;
    else
      return d.doubleValue();
  } 

  /**
   * New method to return a double image directly.
   */
  public synchronized double getdouble( int key )
  {
    return getdouble( new Integer(key) );
  }
}
