/* Set.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.StringBuffer;
import kb7.general.Bug;


/**
 * This class implements a set.
 */
public final class Set
                   implements Cloneable,
                              java.io.Serializable
{
  private static final long serialVersionUID = 1000;


  private Hashtable table = new Hashtable();


  public Set()
  {
  }

  public Set( Hashtable table )
  {
    this.table = table;
  }


  public final synchronized void addElement( Object o )
  {
    this.table.put( o, o );
  }

  public final synchronized void put( Object o )
  {
    this.table.put( o, o );
  }


  public final synchronized void deleteElement( Object o )
  {
    this.table.remove( o );
  }

  public final synchronized void remove( Object o )
  {
    this.table.remove( o );
  }


  public final void unite( Set s2 )
  {
    unite( s2.elements() );
  }

  public final void unite( Enumeration s2_elements )
  {
    while ( s2_elements.hasMoreElements() ) {
      Object s2_x = s2_elements.nextElement();
      addElement( s2_x );
    }
  }

  public final void unite( Vector s2 )
  {
    unite( s2.elements() );
  }


  public final Set union( Set s2 )
  {
    Set result = (Set)this.clone();
    result.unite( s2 );
    return result;
  }


  public final int size()
  {
    return this.table.size();
  }


  public final boolean isEmpty()
  {
    return this.table.size() == 0;
  }


  public final Enumeration elements()
  {
    return this.table.elements();
  }


  public final boolean contains( Object o )
  {
    return this.table.contains(o);
  }


  /*
   * Assumes the set to be a singleton, and returns the only
   * element. 
   */
  public final Object theElement()
  {
    if ( size() == 0 ) 
      throw new Bug( "Set has no elements" );
    else if ( size() > 1 )
      throw new Bug( "Set has more than one element" );
    else 
      return elements().nextElement();
  }


  /*
   * Returns one of the elements. There are a few occasions when this
   * is useful, even for non-singletons.
   */
  public final Object chooseElement()
  {
    if ( size() == 0 ) 
      throw new Bug( "Set has no elements" );
    else 
      return elements().nextElement();
  }


 /**
   * Overrides the Object.clone method, by copying the hashtable.
   */
  public final Object clone()
  {
    return new Set( (Hashtable)this.table.clone() );
  }


  public final String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append( "{" );
    Enumeration keys = this.table.keys();
    while ( keys.hasMoreElements() ) {
      Object key = keys.nextElement();
      sb.append( " " );
      sb.append( key.toString() );
      if ( keys.hasMoreElements() )
        sb.append( "," );
      if ( this.table.size() > 0 )
        sb.append( " " );
    }
    sb.append( "}" );
    return sb.toString();
  }
}
