/* Term.java */


package kb7.structures;

import java.util.Vector;


/**
 * A generic class for representing terms.
 * These are simply nodes of a tree, with
 * the operator and operands both capable of
 * being arbitrary objects.
 */
public class Term
             implements java.io.Serializable
{
  protected Object operator="NONE";

  protected Vector operands = new Vector();

  public Term()
  {}

  public Term( Object operator )
  { this.operator = operator; }

  public Term( Object operator, Object operand )
  { this( operator );
    addOperand( operand );
  }

  public Term( Object operator, Object operand1, Object operand2 )
  { this( operator, operand1 );
    addOperand( operand2 );
  }

  public Term( Object operator, Object operand1, Object operand2, Object operand3 )
  { this( operator, operand1, operand2 );
    addOperand( operand3 );
  }

  public Term( Object operator, Object operand1, Object operand2, Object operand3, Object operand4 )
  { this( operator, operand1, operand2, operand3 );
    addOperand( operand4 );
  }

  public final void setOperator( Object operator )
  { this.operator = operator; }

  public final void setOperands( Vector operands )
  { this.operands = operands; }

  /**
   * Changes the operand at i, expanding the vector if
   * there isn't one there. Use with caution in the latter
   * case!
  */
  public final void setOperand( int i, Object operand )
  { if ( this.operands.size() >= i + 1 )
      this.operands.setElementAt( operand, i );
    else { 
      Vector copy = (Vector)this.operands.clone();
      for ( int j=this.operands.size(); j<i; j++ )
        copy.addElement( "NONE" );
      copy.addElement( operand );
      this.operands = copy;
    }
  }

  public final void addOperand( Object operand )
  { this.operands.addElement( operand ); }

  public final void addOperands( Vector operands )
  { for ( int i=0; i<operands.size(); i++ )
      addOperand( operands.elementAt(i) );
  }


  public final Object getOperator()
  { return this.operator; }

  public final Vector getOperands()
  { return this.operands; }

  public final Object getOperand( int n )
  { return this.operands.elementAt(n); }


  public String toString()
  {
    return "[Term " + this.operator + " " + this.operands + "]";
  }
}
