/* BoundedFixedPointComboField.java */


package kb7.swing;


/**
 * This class implements a fixed-point combo field with upper and
 * lower bounds on its values. It does this in the same way
 * that BoundedFixedPointField does FixedPointField.
 */  
public class BoundedFixedPointComboField extends FixedPointComboField  
{
  public static void main( String argv[] )
  {
    BoundedFixedPointComboField bfpcf = 
      new BoundedFixedPointComboField( "1.5", 
                             new MenuOptions().add( "One", "1" ).
                                               add( "Two", "2" ).
                                               add( "Three", "3" ).
                                               add( "pi", "3.1416" ).
                                               add( "Halve it", "*0.5" ). 
                                               add( "Double it", "*2" ).  
                                               add( "Triple it", "*3" ).
                                               add( "Add 1", "+1" ).
                                               add( "Add 2", "+2" )
                           );
    bfpcf.setColumns(5);
    bfpcf.setMin( 1.5 );
    bfpcf.setMax( 200.5 );
    bfpcf.setAfter( 4 );
    bfpcf.setToolTipText( "Number between 1.5 and 200.5" );    

    javax.swing.JPanel p = new javax.swing.JPanel();
    p.add( bfpcf );

    javax.swing.JFrame f = new javax.swing.JFrame();
    f.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    f.getContentPane().add( p );
    f.setSize( 400, 400 );
    f.pack();
    f.setVisible( true );
  }


  public BoundedFixedPointComboField()
  {
    super();
    this.constraints = new RealRangeConstraint();
  }

  public BoundedFixedPointComboField( MenuOptions options )
  {
    this();
    setOptions( options );
  }

  public BoundedFixedPointComboField( String text )
  { 
    this();
    setTextValue( text ); 
  }

  public BoundedFixedPointComboField( String text, MenuOptions options )
  {
    this( text );
    setOptions( options );
  }


  public final void setMin( double min )
  { ((RealRangeConstraint)this.constraints).setMin(min); }

  public final double getMin()
  { return ((RealRangeConstraint)this.constraints).getMin(); }  

  public final void setMax( double max )
  { ((RealRangeConstraint)this.constraints).setMax(max); }

  public final double getMax()
  { return ((RealRangeConstraint)this.constraints).getMax(); }
}

