/* BoundedRealField.java */


package kb7.swing;


/**
 * This field extends RealField by producing an error
 * message if the integer entered exceeds a minimum
 * or maximum limit. We use it in the client-side version
 * of WOM.
 *
 * It adds methods setMin(d), getMin(), setMax(d),
 * getMax(), to RealField. By default, the bounds are
 * -Real.MAX_VALUE and Real.MAX_VALUE.
 *
 * Note that setdoubleValue will report an error if its
 * argument is outside the bounds. This is because it relies
 * on the superclass's implementation, which is to convert
 * the number to text and call setTextValue on that.
 *
 * IMPLEMENTATION
 *
 * The class extends RealField, using RealRangeConstraint.
 */
public class BoundedRealField extends RealField
{
  public static void main( String[] argv )
  {
    BoundedRealField brf1 = new BoundedRealField();
    brf1.setdoubleValue(1);
    brf1.setColumns(5);
    brf1.setMin(1.5);
    brf1.setMax(10.5);
    brf1.setLenient( false );
    try {
      brf1.setdoubleValue( 11.5 );
    }
    catch ( Throwable e ) {
      System.out.println( "Result of setting field outside range: " + e );
    }

    BoundedRealField brf2 = new BoundedRealField();
    brf2.setTextValue("2.6");
    brf2.setColumns(5);  
    brf2.setMin(2.5);
    brf2.setMax(20.5);

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( brf1 );
    panel.add( brf2 );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public BoundedRealField()
  {
    super();
    this.constraints = new RealRangeConstraint();
  }

  public BoundedRealField( double value )
  { 
    this();
    setdoubleValue(value); 
  }

  public BoundedRealField( String text )
  { 
    this();
    setTextValue(text); 
  }


  public final void setMin( double min )
  { ((RealRangeConstraint)this.constraints).setMin(min); }

  public final double getMin()
  { return ((RealRangeConstraint)this.constraints).getMin(); }  

  public final void setMax( double max )
  { ((RealRangeConstraint)this.constraints).setMax(max); }

  public final double getMax()
  { return ((RealRangeConstraint)this.constraints).getMax(); }
} 
