/* CheckBox.java */


package kb7.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.event.EventListenerList;
import javax.swing.text.Document;
import kb7.general.Bug;


/**
 *   setValue(Object), getValue()
 *   setintValue(int), getintValue()
 *   setbooleanValue(boolean), getbooleanValue()
 *
 *   addValueListener( ValueListener ), removeValueListener( ValueListener )
 * These add and remove instances of ValueListener. 
 */
public class CheckBox extends javax.swing.JCheckBox 
{
  private EventListenerList value_listeners = new EventListenerList();

  private String false_option, true_option;


  public static void main( String[] argv )
  {
    final CheckBox cb1 = new CheckBox( "CheckBox 1" );
    cb1.setintValue(1);

    final CheckBox cb2 = new CheckBox();
    cb2.setText( "CheckBox 2" );

    cb1.addValueListener( new ValueListener() {
                            public void valueAccepted( ValueAcceptedEvent ev )
                            {
                              System.out.println( "cb1 : integer value = " + cb1.getintValue() );
                            }
                        } ); 

    cb2.addValueListener( new ValueListener() {
                            public void valueAccepted( ValueAcceptedEvent ev )
                            {
                              System.out.println( "cb2 : boolean value = " + cb2.getbooleanValue() );
                            }
                        } );

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( cb1 );
    panel.add( cb2 );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public CheckBox()
  { 
    super(); 
    addListener();
  }

  public CheckBox( String text )
  { 
    super( text ); 
    addListener();
  }

  private final void addListener()
  {
    addActionListener( new ActionListener() {
                         public void actionPerformed( ActionEvent ev ) {}
    } );
  }


  public void setTextValue( String value )
  {
    if ( value.equals("1") 
      || value.equalsIgnoreCase("yes") 
      || value.equalsIgnoreCase("y")
      || value.equalsIgnoreCase("true") 
      || value.equalsIgnoreCase("t") )
      setbooleanValue( true );
    else if ( value.equals("0") 
           || value.equalsIgnoreCase("no")
           || value.equalsIgnoreCase("n") 
           || value.equalsIgnoreCase("false") 
           || value.equalsIgnoreCase("f") ) 
      setbooleanValue( false ); 
  }

  public String getTextValue()
  { 
    return getValue().toString();
  }

  
  /**
   * If the value is an Boolean, use its booleanvalue. Otherwise,
   * convert to string, and hope the result makes sense 
   * to setTextValue.
   */
  public void setValue( Object value )
  {
    if ( value instanceof Boolean )
      setbooleanValue( ((Boolean)value).booleanValue() );
    else
      setTextValue( value.toString() );
  }

  public Object getValue()
  { return new Boolean(getbooleanValue()); }


  public void setbooleanValue( boolean value )
  { 
    setSelected( value );
  }

  public boolean getbooleanValue()
  {
    return isSelected();
  }

  public void setintValue( int value )
  { 
    if ( value == 0 ) setbooleanValue( false );
    else if ( value ==1 ) setbooleanValue( true );
    else throw new Bug( "Illegal value: " + value );
  }

  public int getintValue()
  {
    return ( getbooleanValue() ? 1 : 0 );
  }


  public final void addValueListener( ValueListener vl )
  { this.value_listeners.add( ValueListener.class, vl ); }

  public final void removeValueListener( ValueListener vl )
  { this.value_listeners.remove( ValueListener.class, vl ); }

  protected final void fireValueEvent( ValueAcceptedEvent ev )
  {
    Object[] listeners = this.value_listeners.getListenerList();
    for ( int i=0; i<this.value_listeners.getListenerCount(); i+=2 ) {
      if ( listeners[i] == ValueListener.class ) 
        ( (ValueListener)listeners[i+1] ).valueAccepted( ev );
    }
  }
}
