/* FixedPointFieldFormat.java */


package kb7.swing;

import java.awt.Toolkit;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument; 
import kb7.strings.Strings;


public final class FixedPointFieldFormat extends FieldFormat
{
  private static int DIGITS_ONLY = 0;
  private static int DIGITS_AND_POINT = 1;  

  private Toolkit toolkit;

  private int after = 2;
  // The maximum number of digits allowed after the
  // point.


  public FixedPointFieldFormat()
  {
    this.toolkit = Toolkit.getDefaultToolkit();
    this.document = new FixedPointDocument();
  }


  public String checkText( String text )
  {
    if ( text.equals(".") )
      return "Invalid fixed point number: needs digits.";
    else if ( text.equals("") )
      return "Invalid fixed point number: empty string.";
    else
      return null;
  }


  public final String standardiseText( String text )
  {
    if ( text.startsWith(".") )
      text = "0" + text;

    /* Add the extra digits after the point. */
    if ( text.indexOf(".") == -1 ) 
      text = text + "." + Strings.multiply("0",this.after);
    else {
      int point_position = text.indexOf(".");
      int digits_after_point = (text.length() - point_position) - 1;
      int surplus_digits = digits_after_point - this.after;
      if ( surplus_digits > 0 ) 
        text = text.substring( 0, text.length() - surplus_digits );
      else if ( surplus_digits < 0 ) 
        text = text + Strings.multiply( "0", this.after-digits_after_point );
    }

    /* Remove the point if after=0. */
    if ( this.after == 0 )
      text = Strings.replace( text, ".", "" );
    
    return text;
  }


  public final void setAfter( int after )
  { this.after = after; }

  public final int getAfter()
  { return this.after; }


  private final class FixedPointDocument extends PlainDocument 
  {
    private int state = DIGITS_ONLY;
    private int after_point_digits = 0;

    public void insertString( int offs, String str, AttributeSet a ) 
                throws BadLocationException 
    {
      char[] source = str.toCharArray();
      char[] result = new char[source.length];
      int j = 0;
      int state = this.state;

      for ( int i=0; i<result.length; i++ ) {
        if ( Character.isDigit(source[i]) )
          result[j++] = source[i];
        else if ( source[i] == '.' ) {
          if ( state == DIGITS_ONLY ) {
            result[j++] = source[i]; 
            state = DIGITS_AND_POINT; 
          }
          else 
;//            toolkit.beep();
        }
        else
;//          toolkit.beep();
      }
      super.insertString( offs, new String(result, 0, j), a );
      this.state = state;
    }

    public void remove( int offs, int len ) 
                throws BadLocationException 
    {
      super.remove( offs, len );

      /* Recalculate new state. 
      */
      int point_position = getText(0,getLength()).indexOf(".");
      if ( point_position == -1 )
        this.state = DIGITS_ONLY;
      else {
        this.state = DIGITS_AND_POINT;
        this.after_point_digits = (getText(0,getLength()).length() - point_position) - 1;
      }
    }
  }
}
