/* IntegerRangeConstraint.java */


package kb7.swing;

import kb7.structures.Pair;


/**
 * This class represents bounds on the value of an integer field.
 * Use it with CheckingField.
 */
final class IntegerRangeConstraint extends FieldConstraint
{
  private int min=Integer.MIN_VALUE, max=Integer.MAX_VALUE;

  public IntegerRangeConstraint()
  { }

  public IntegerRangeConstraint( int min, int max )
  {
    this.min = min;
    this.max = max;
  }

  public final Pair check( String s, boolean is_lenient )
  {
    String obligation = ( is_lenient ? "is recommended to be" : "must be" );
    int n = Integer.parseInt( s );
    if ( n < this.min ) 
      return new Pair( "Integer out of range: it " + obligation + " " + this.min + " or more.", this.min+"" );
    else if ( n > this.max )
      return new Pair( "Integer out of range: it " + obligation + " " + this.max + " or less.", this.max+"" );
    else
      return null;
  }

  public void setMin( int min )
  { this.min = min; }

  public int getMin()
  { return this.min; }  

  public void setMax( int max )
  { this.max = max; }

  public int getMax()
  { return this.max; }
}
