/* TextField.java */


package kb7.swing;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.event.EventListenerList;
import javax.swing.text.Document;


/**
 * This field extends JTextField with a few methods that
 * we find handy in WOM. We use it as the base class for
 * IntegerField etc.
 *
 * The added methods are:
 *
 *   setValue(Object), getValue()
 * These set and get the text, but are defined to accept
 * and return an Object. This is because we shall want to
 * extend them in subclasses.
 *
 *
 *  updateValueFromAction( String op, Object value )
 * Used in implementing combo fields, and called after op has been
 * applied, but before value is set. 
 *
 *  addValueListener( ValueListener ), removeValueListener( ValueListener )
 * These add and remove instances of ValueListener. A ValueAcceptedEvent
 * is thrown by a CheckingField (subclass of TextField) when 
 * not only has the user entered text into the field, but the
 * text has been checked for correct syntax and other contraints,
 * such as minimum and maximum bounds. If it satisfies all these,
 * we throw a ValueAcceptedEvent.
 */
public class TextField extends javax.swing.JTextField 
{
  private EventListenerList value_listeners = new EventListenerList();


  public static void main( String argv[] )
  {
    javax.swing.JPanel p = new javax.swing.JPanel();
    TextField tf = new TextField();
    tf.setColumns(5);
    tf.setTextValue("abcd");
    p.add( tf );

    javax.swing.JFrame f = new javax.swing.JFrame();
    f.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    f.getContentPane().add( p );
    f.setSize( 200, 200 );
    f.pack();
    f.setVisible( true );
  }


  public TextField()
  { super(); }

  public TextField( String text )
  { super(text); }

  protected TextField( Document d )
  { super(d," ",10); }


  public void setTextValue( String value )
  { setText(value); }

  public String getTextValue()
  { return getText(); }


  public void setValue( Object value )
  { setTextValue( (String)value ); }

  public Object getValue()
  { return getTextValue(); }


  /**
   * May be overridden by subclasses.
   */
  protected void updateValueFromAction( String op, Object value )
  {
System.out.println( "TextField.updateValueFromAction" );
    setValue( value );
  }


  public final void addValueListener( ValueListener vl )
  { this.value_listeners.add( ValueListener.class, vl ); }

  public final void removeValueListener( ValueListener vl )
  { this.value_listeners.remove( ValueListener.class, vl ); }

  protected final void fireValueEvent( ValueAcceptedEvent ev )
  {
    Object[] listeners = this.value_listeners.getListenerList();
    for ( int i=0; i<this.value_listeners.getListenerCount(); i+=2 ) {
      if ( listeners[i] == ValueListener.class ) 
        ( (ValueListener)listeners[i+1] ).valueAccepted( ev );
    }
  }
}
