/* BoundedFixedPointField.java */


package kb7.swing;


/**
 * This field extends FixedPointField by producing an error
 * message if the number entered exceeds a minimum
 * or maximum limit. We use it in the client-side version
 * of WOM.
 *
 * It adds methods setMin(i), getMin(), setMax(i),
 * getMax(), to FixedPointField. By default, the bounds are
 * -Double.MAX_VALUE and Double.MAX_VALUE.
 *
 * Note that setdoubleValue will report an error if its
 * argument is outside the bounds. This is because it relies
 * on the superclass's implementation, which is to convert
 * the number to text and call setTextValue on that.
 *
 * IMPLEMENTATION
 *
 * The class extends FixedPointField, using RealRangeConstraint.
 */
public class BoundedFixedPointField extends FixedPointField
{
  public static void main( String[] argv )
  {
    BoundedFixedPointField bfpf1 = new BoundedFixedPointField();
    bfpf1.setdoubleValue(1.6);
    bfpf1.setColumns(5);
    bfpf1.setMin(1.5);
    bfpf1.setMax(10.5);

    try {
      bfpf1.setdoubleValue( 11.0 );
    }
    catch ( Throwable e ) {
      System.out.println( "Result of setting field outside range: " + e );
    }

    BoundedFixedPointField bfpf2 = new BoundedFixedPointField();
    bfpf2.setTextValue("2.6");
    bfpf2.setColumns(5);  
    bfpf2.setMin(2.5);
    bfpf2.setMax(20.5);

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( bfpf1 );
    panel.add( bfpf2 );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public BoundedFixedPointField()
  {
    super();
    this.constraints = new RealRangeConstraint();
  }

  public BoundedFixedPointField( int value )
  { 
    this();
    setdoubleValue(value); 
  }

  public BoundedFixedPointField( String text )
  { 
    this();
    setTextValue(text); 
  }


  public final void setMin( double min )
  { ((RealRangeConstraint)this.constraints).setMin(min); }

  public final double getMin()
  { return ((RealRangeConstraint)this.constraints).getMin(); }  

  public final void setMax( double max )
  { ((RealRangeConstraint)this.constraints).setMax(max); }

  public final double getMax()
  { return ((RealRangeConstraint)this.constraints).getMax(); }
} 
