/* BoundedIntegerField.java */


package kb7.swing;


/**
 * This field extends IntegerField by producing an error
 * message if the integer entered exceeds a minimum
 * or maximum limit. We use it in the client-side version
 * of WOM.
 *
 * It adds methods setMin(i), getMin(), setMax(i),
 * getMax(), to IntegerField. By default, the bounds are
 * Integer.MIN_VALUE and Integer.MAX_VALUE.
 *
 * Note that setintValue will report an error if its
 * argument is outside the bounds. This is because it relies
 * on the superclass's implementation, which is to convert
 * the integer to text and call setTextValue on that.
 *
 * IMPLEMENTATION
 *
 * The class extends IntegerField, using an IntegerRangeConstraint.
 */
public class BoundedIntegerField extends IntegerField
{
  public static void main( String[] argv )
  {
    BoundedIntegerField bif1 = new BoundedIntegerField();
    bif1.setintValue(1);
    bif1.setColumns(5);
    bif1.setMin(1);
    bif1.setMax(10);
    bif1.setLenient( false );
    try {
      bif1.setintValue( 11 );
    }
    catch ( Throwable e ) {
      System.out.println( "Result of setting field outside range: " + e );
    }

    BoundedIntegerField bif2 = new BoundedIntegerField();
    bif2.setTextValue("2");
    bif2.setColumns(5);  
    bif2.setMin(2);
    bif2.setMax(20);

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( bif1 );
    panel.add( bif2 );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public BoundedIntegerField()
  {
    super();
    this.constraints = new IntegerRangeConstraint();
  }

  public BoundedIntegerField( int value )
  { 
    this();
    setintValue(value); 
  }

  public BoundedIntegerField( String text )
  { 
    this();
    setTextValue(text); 
  }


  public final void setMin( int min )
  { ((IntegerRangeConstraint)this.constraints).setMin(min); }

  public final int getMin()
  { return ((IntegerRangeConstraint)this.constraints).getMin(); }  

  public final void setMax( int max )
  { ((IntegerRangeConstraint)this.constraints).setMax(max); }

  public final int getMax()
  { return ((IntegerRangeConstraint)this.constraints).getMax(); }
} 
