/* FieldFormat.java */


package kb7.swing;

import java.awt.Toolkit;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument; 
import kb7.strings.Strings;


/**
 * This class encapsulates information about the 
 * format of text that can be entered into a field.
 * It contains two components. One is a Document.
 * This defines insertString and remove methods
 * which validate text as it's being typed. They
 * may, for example, reject non-digits from being
 * input. The other is the standardiseText method.
 * This operates on the entire field's worth of
 * text once the user has pressed ENTER, and returns
 * it in some standard form which will replace
 * the original. For example, in a fixed-point field,
 * standardiseText may truncate the substring 
 * after the decimal point or extend it with zeroes
 * to make it specified length.
 * 
 * This class is abstract: its subclasses represent
 * specific formats. Use them with CheckingField.
 * This defines a createDefaultDocument method which
 * returns the subinstance's document (accessible
 * via getDocument); calls checkText to check the text
 * once the user has pressed ENTER; and calls standardiseText 
 * once checkText has verified that the entered string 
 * is valid.
 */
public abstract class FieldFormat
{
  protected PlainDocument document;

  public abstract String standardiseText( String text );

  public abstract String checkText( String text );

  public final PlainDocument getDocument()
  { return this.document; }
}

