/* IntegerComboBox.java */


package kb7.swing;

import javax.swing.*; 
import javax.swing.text.*; 

import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;


/**
 * This ComboBox only allows digits to be typed into it.
 * Methods added to TextComboBox are
 *   getintValue() and setintValue(i),
 * which return and set the text as an integer. 
 * 
 * IMPLEMENTATION
 *
 * I would have preferred getintValue and setintValue to be called 
 * getValue and setValue. However, I might want to introduce a getValue
 * with return type String into the superclass, as I do in 
 * kb7.wom.client.TextComboBox. Then there'd be a clash, because the
 * getValue here would have a different return type from that in the
 * superclass. The same thing happens in kb7.wom.client.TextComboBox
 * versus kb7.wom.client.IntegerComboBox.
 *
 * !!! The present version doesn't use an IntegerField. I'm waiting to
 * find out how to use a field other than a JTextField in a combo box.
 * Also, the relation between translated and untranslated values needs
 * rethinking.
 */
public class IntegerComboBox extends TextComboBox 
{
  public static void main( String[] argv )
  {
    final IntegerComboBox cb = new IntegerComboBox();
    cb.addItem( new MenuOption( "One", "1" ) );
    cb.addItem( new MenuOption( "Two", "2" ) );
    cb.addItem( new MenuOption( "Three", "3" ) );

    cb.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        System.out.println( "User selected text " + cb.getTextValue() );
        System.out.println( "Value = " + cb.getintValue() );
        System.out.println();
      }
    });

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.setLayout( new java.awt.FlowLayout() );
    panel.add( cb );

    final javax.swing.JTextField key = new javax.swing.JTextField();
    key.setColumns(10);
    panel.add( key );
    key.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        System.out.println( "Setting key to " + key.getText() );
        cb.setTextValue( key.getText() );
        System.out.println();
      }
    });

    final IntegerField value = new IntegerField();
    value.setColumns(10);
    panel.add( value );
    value.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        System.out.println( "Setting value to " + value.getintValue() );
        cb.setintValue( value.getintValue() );
        System.out.println();
      }
    });

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.setSize( 300, 400 );
    frame.setContentPane( panel );
    frame.pack();
    frame.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    frame.setVisible( true );  
  }


  public IntegerComboBox() 
  {  
    super( new IntegerField() );
  }

  public IntegerComboBox( int value )
  {
    this();
    setintValue( value );
  }

  public IntegerComboBox( String text )
  {
    this();
    setTextValue( text );
  }  


  public int getintValue() 
  {
    return Integer.parseInt( getValue().toString() );
  }

  public void setintValue( int value ) 
  {
    this.setValue( value + "" );
  }
}
