/**
  NAME:         Meter.java
                Copyright (c) 1996 Liam Relihan. All rights reserved.
  PURPOSE:

  AUTHOR:       Liam Relihan
                <relihanl@ul.ie>
  SCCS:         %M%        %R%.%L% %D%
  DATE:         15-May-1996
  PARAMETERS:
  CALLED BY:
  MODIFIED:     10-June-96     LOR     Added control for Panel color
  BUGS:
*/


package kb7.swing; 

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.JPanel;


public abstract class Meter extends JPanel
{
   protected double lowestValue=0.0;
   protected double highestValue=100.0;
   protected double scaleGrad=10.0;
   protected double value = 0.0;
   protected Font captionFont;
   protected Font numbersFont;
   protected String caption="Uncaptioned Meter";
   protected Image offScreenImage;
   protected Dimension offScreenSize;
   protected Graphics offScreenGraphics;
   protected Color panelColor=Color.lightGray;


   /**  Constructor for Meter class.
    * @arg lo     lowest value the meter can indicate
    * @arg hi     highest value the meter can indicate
    * @arg grad   graduations in which scale is marked
    * @arg val    initial value
    */
   public Meter(double lo,double hi,double grad,double val) 
   {
     lowestValue=lo;
     highestValue=hi;
     scaleGrad=grad;
     value=val;
     // default fonts
     captionFont = new Font("Helvetica", Font.PLAIN, 15);
     numbersFont = new Font("Helvetica", Font.PLAIN, 10);
   }


   /**  set the valid range of the  meter
    * @arg theMinimum     lowest value the meter can indicate
    * @arg theMaximum     highest value the meter can indicate
    */
   public void setRange(int theMinimum, int theMaximum) 
   {
      lowestValue = theMinimum;
      highestValue = theMaximum;
      repaint();
   }


   /**  set the value that meter should indicate
    * @arg theValue     the new value of the meter
    */
   public void setValue(double theValue)
   {
      value = theValue;
      if (value < lowestValue)
         value = lowestValue;
      if (value > highestValue)
         value = highestValue;
      repaint();
   }


   /** get the minimum value that the meter can indicate */
   public double getMinimum() { return lowestValue; }


   /** get the maximum value that the meter can indicate */
   public double getMaximum() { return highestValue; }


   /** get the value that the meter is indicating */
   public double getValue()   { return value; }


   /** sets the panel color */
   public void setPanelColor(Color theColor)
   {
      panelColor = theColor;
      repaint();
   }


   /** gets the panel color */
   public Color getPanelColor()
   {
      return(panelColor);
   }


   /** sets the caption */
   public void setCaption(String theCaption)
   {
      caption = theCaption;
      repaint();
   }


   /** sets the font of the caption */
   public void setCaptionFont(Font theFont)
   {
      captionFont = theFont;
      repaint();
   }


   /** sets the font of the readings */
   public void setNumbersFont(Font theFont)
   {
      numbersFont = theFont;
      repaint();
   }


   /** paint the meter on the foreground*/
   public synchronized void paint(Graphics g) 
   {
     Dimension d = this.size();

     if((offScreenImage == null) || (d.width != offScreenSize.width) ||
	(d.height != offScreenSize.height)) {
       offScreenImage = createImage(d.width, d.height);
       offScreenSize = d;
       offScreenGraphics = offScreenImage.getGraphics();
    }

     drawMeter(offScreenGraphics);
     g.drawImage(offScreenImage, 0, 0, null);

   }

   /** redefines inherited method to exclude screen clear */
   public void update(Graphics g) 
   {
     paint(g);
   }

   /* draws the gauge in the background. This allows the reduction of
    * flicker.
    */
   protected abstract void drawMeter(Graphics g);


   public abstract Dimension minimumSize();

   public abstract Dimension preferredSize();
}


class ScalablePlane extends Object implements Cloneable
{
  public static double x_scale;
  public static double y_scale;

  public ScalablePlane(int sp_x, int sp_y, int actual_x, int actual_y) 
  {
    x_scale=(double)actual_x/(double)sp_x;
    y_scale=(double)actual_y/(double)sp_y;
  }


  public int x(int sp_x)
  {return (int)((double)sp_x*x_scale);}

  public int y(int sp_y)
  {return (int)((double)sp_y*y_scale);}
}
