/**
 *
  NAME:         Needle.java
                Copyright (c) 1996 Liam Relihan. All rights reserved.
  PURPOSE:      This class implements a simple swing needle widget.
                Inherits from Meter.class.
  AUTHOR:       Liam Relihan
                <relihanl@ul.ie>
  SCCS:         %M%        %R%.%L% %D%
  DATE:         15-May-1996
  PARAMETERS:
  CALLED BY:
  MODIFIED:
  BUGS:
 */
package kb7.swing; 

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Vector;


public class Needle extends Meter
{
  private double radiansPerDegree = Math.PI / 180;
  private final double MIN_NEEDLE_ANGLE=10.0;
  private final double MAX_NEEDLE_ANGLE=170.0;
  public Vector zones;
  protected Color needleColor=Color.black;


  public static void main( String[] argv )
  {
    Needle needle = new Needle( 0, 10, 1, 3.4 ); 
    needle.setZone( "Danger", 9, 10, Color.red );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( needle );
    frame.setSize( 100, 200 );
    frame.setVisible( true );
  }


   /**  Constructor for Needle class.
    * @arg lo     lowest value the meter can indicate
    * @arg hi     highest value the meter can indicate
    * @arg grad   graduations in which scale is marked
    * @arg val    initial value
    */
  public Needle(double lo,double hi,double grad,double val)
  {
    super(lo,hi,grad,val);
    zones=new Vector();
  }


   /** set a color zone, e.g. a red "Danger" zone
    * @arg zoneName     the name of the zone, e.g. "danger zone"
    * @arg lowerBound   the lower bound of the zone
    * @arg upperBound   the upper bound of the zone
    * @arg color        the color of the zone
    */
  public void setZone(String zoneName,
		       double lowerBound,
		       double upperBound,
		       Color color)
  {
    if (lowerBound<lowestValue) lowerBound=lowestValue;
    if (upperBound<lowestValue) upperBound=lowestValue;
    if (lowerBound>highestValue) lowerBound=highestValue;
    if (upperBound>highestValue) upperBound=highestValue;
    zones.addElement(new MeterZone(zoneName,lowerBound,upperBound,color));
    repaint();
  }


   /** 
    * set the needle color
    * @arg theColor   the color of the needle
    */
  public void setNeedleColor(Color theColor)
  {
    needleColor=theColor;
    repaint();
  }


   /** get the needle color */
   public Color getNeedleColor()
   {
     return(needleColor);
   }


   /* draws the gauge in the background. This allows the reduction of
    * flicker.
    */
   protected void drawMeter(Graphics g) 
   {
     int width=1000;
     int height=1000;
     int needle_length=width/3;
     int mark_length=width/30;
     int width_of_scale=needle_length*2;
     int meter_border=20;
     int pivot_x=width/2;
     int pivot_y=(int)((double)height*0.7);
     int caption_y=(int)((double)height*0.85);
     int coil_width=width/30;
     int coil_height=height/30;

     ScalablePlane sp=new ScalablePlane(width,
					height,
					bounds().width,
					bounds().height);

     g.setColor(panelColor);

     g.fill3DRect(sp.x(meter_border),
		  sp.y(meter_border),
		  sp.x(width-meter_border*2),
		  sp.y(height-meter_border*2),
		  true);

     // draw the colored zones
     for(int j=0;j<zones.size();j++){
       MeterZone cur_zone= (MeterZone)zones.elementAt(j);

       double start_angle=valueToAngle(cur_zone.getZoneLow());

       double end_angle=valueToAngle(cur_zone.getZoneHigh());

       g.setColor(cur_zone.getZoneColor());
       g.fillArc(sp.x(pivot_x-needle_length),sp.y(pivot_y-needle_length),
		 sp.x(needle_length*2),sp.y(needle_length*2),
		 (int)start_angle,
		 (int)(end_angle-start_angle));
     }


     g.setColor(Color.black);
     // draw graduations
     for(double i=lowestValue;i<=highestValue;i=i+scaleGrad) {
       double mark_angle=valueToAngle(i);

       mark_angle=360-mark_angle;
       mark_angle=mark_angle * radiansPerDegree;

       int mark_x1=sp.x(pivot_x)+
	 (int)(Math.cos(mark_angle)*(double)sp.x(needle_length));
       int mark_y1=sp.y(pivot_y)+
	 (int)(Math.sin(mark_angle)*(double)sp.y(needle_length));

       int mark_x2=sp.x(pivot_x)+
	 (int)(Math.cos(mark_angle)*(double)sp.x(needle_length+mark_length));
       int mark_y2=sp.y(pivot_y)+
	 (int)(Math.sin(mark_angle)*(double)sp.y(needle_length+mark_length));

       g.drawLine(mark_x1,mark_y1,mark_x2,mark_y2);

       int text_x=sp.x(pivot_x)+
	 (int)(Math.cos(mark_angle)*
	       (double)sp.x(needle_length+mark_length*2));
       int text_y=sp.y(pivot_y)+
	 (int)(Math.sin(mark_angle)*
	       (double)sp.y(needle_length+mark_length*2));
       g.setFont(numbersFont);
       int mid_string=(g.getFontMetrics().stringWidth(String.valueOf(i))/2);
       g.drawString(String.valueOf(i),text_x-mid_string,text_y);
     }

     // draw the needle
     double angle=valueToAngle(value);
     angle=360-angle;
     angle=angle * radiansPerDegree;

     g.setColor(needleColor);
     g.drawLine(sp.x(pivot_x),
		sp.y(pivot_y),
		sp.x(pivot_x)+
		(int)(Math.cos(angle)*(double)sp.x(needle_length)),
		sp.y(pivot_y)+
		(int)(Math.sin(angle)*(double)sp.y(needle_length)));

     // draw coil
     g.fillRect(sp.x(pivot_x-coil_width/2),
		sp.y(pivot_y-coil_height/2),
		sp.x(coil_width),
		sp.y(coil_height));

     // draw caption
     g.setFont(captionFont);
     int caption_length=g.getFontMetrics().stringWidth(caption);
     g.drawString(caption,sp.x(pivot_x)-(caption_length/2),sp.y(caption_y));
   }

   public Dimension minimumSize()
   { return new Dimension(250, 150); }

   public Dimension preferredSize() 
   { return minimumSize(); }

   /** 
    * calculates the angle (in degrees) for a given value 
    */
   private double valueToAngle(double value)
   {
     double degrees_per_unit=(MAX_NEEDLE_ANGLE-MIN_NEEDLE_ANGLE)/
       (highestValue-lowestValue);
     return MAX_NEEDLE_ANGLE-((value-lowestValue)*degrees_per_unit);
   }
}


class MeterZone extends Object implements Cloneable 
{
  private String zonename;
  private double lo;
  private double hi;
  private Color zonecolor;

  public MeterZone(String name_val,double lo_val, double hi_val, Color color)
  {
    zonename=name_val;
    lo=lo_val;
    hi=hi_val;
    zonecolor=color;
  }

  public String getZoneName()
  { return zonename;}

  public double getZoneLow()
  { return lo;}

  public double getZoneHigh()
  { return hi;}

  public Color getZoneColor()
  { return zonecolor;}
}
