/* RealFieldFormat.java */


package kb7.swing;

import java.awt.Toolkit;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument; 
import kb7.strings.Strings;


public final class RealFieldFormat extends FieldFormat
{
  private static int DIGITS_ONLY = 0;
  private static int DIGITS_AND_POINT = 1;
  
  private Toolkit toolkit;


  public RealFieldFormat() 
  {  
    this.toolkit = Toolkit.getDefaultToolkit();
    this.document = new RealDocument();
  }


  public String checkText( String text )
  {
    if ( text.equals(".") )
      return "Invalid real number: needs digits.";
    else if ( text.equals("") )
      return "Invalid real number: empty string.";
    else 
      return null;
  }


  public final String standardiseText( String text )
  {
    if ( text.endsWith(".") )
      text += "0";
    else if ( text.startsWith(".") )
      text = "0" + text;
    return text;
  }


  private final class RealDocument extends PlainDocument 
  {
    private int state = DIGITS_ONLY;

    public void insertString( int offs, String str, AttributeSet a ) 
                throws BadLocationException 
    {
      char[] source = str.toCharArray();
      char[] result = new char[source.length];
      int j = 0;
      int state = this.state;

      for ( int i=0; i<result.length; i++ ) {
        if ( Character.isDigit(source[i]) )
          result[j++] = source[i];
        else if ( source[i] == '.' ) {
          if ( state == DIGITS_ONLY ) {
            result[j++] = source[i]; 
            state = DIGITS_AND_POINT; 
          }
          else 
;//            toolkit.beep();
        }
        else
;//          toolkit.beep();
      }
      super.insertString( offs, new String(result, 0, j), a );
      this.state = state;
    }

    public void remove( int offs, int len ) 
                throws BadLocationException 
    {
      super.remove( offs, len );
      if ( getText(0,getLength()).indexOf(".") == -1 )
        this.state = DIGITS_ONLY;
      else
        this.state = DIGITS_AND_POINT; 
    }
  }
}

