/* RealRangeConstraint.java */


package kb7.swing;

import kb7.structures.Pair;


/**
 * This class represents bounds on the value of a real field.
 * Use it with CheckingField.
 */
public final class RealRangeConstraint extends FieldConstraint
{
  private double min=-Double.MAX_VALUE, max=Double.MAX_VALUE;

  public RealRangeConstraint()
  { }

  public RealRangeConstraint( double min, double max )
  {
    this.min = min;
    this.max = max;
  }

  public final Pair check( String s, boolean is_lenient )
  {
    String obligation = ( is_lenient ? "is recommended to be" : "must be" );
    double d = new Double( s ).doubleValue();
    if ( d < this.min )
      return new Pair( "Number out of range: it " + obligation + " " + this.min + " or more.", this.min+"" );
    else if ( d > this.max )
      return new Pair( "Number out of range: it " + obligation + " " + this.max + " or less.", this.max+"" );
    else
      return null;
  }

  public void setMin( double min )
  { this.min = min; }

  public double getMin()
  { return this.min; }  

  public void setMax( double max )
  { this.max = max; }

  public double getMax()
  { return this.max; }
}
