/* Thermometer.java */

package kb7.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.lang.*;
import java.util.*;


/**
 * <P>
 * This class implements a simple swing thermometer widget. Inherits from
 * Meter.class. This might be useful in instrumentation panels, etc.
 * </P>
 *
 * <P>
 * Modifications:
 * <TABLE BORDER =1 >
 * <TR><TH>Date of Modification<TH>Developer<TH>Description
 *
 * <TR>
 * <TD>08-04-1997 <TD>LOR <TD>Changed the formatting of comments
 *                            Added needle drawing changes kindly supplied by
 *                            David Weisman &lt;weisman@grimm.osf.org&gt;
 * </TABLE>
 *
 * @author Liam Relihan <liam.relihan@ul.ie>
 * @version %I%
 */
public class Thermometer extends Meter 
{
  public static void main( String[] argv )
  {
    Thermometer t = new Thermometer( 0, 10, 1, 3.4 ); 

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( t );
    frame.setSize( 100, 200 );
    frame.setVisible( true );
  }


  /**  Constructor for Thermometer class.
   * @arg lo     lowest value the thermometer can indicate
   * @arg hi     highest value the thermometer can indicate
   * @arg grad   graduations in which scale is marked
   * @arg val    initial value
   */
  public Thermometer(double lo,double hi,double grad,double val)
  {
    super(lo,hi,grad,val);
  }


  /** 
   * Draw the Thermometer with mercury 
   */
  protected void drawMeter(Graphics g) 
  {
    Color forecolor;
    int width=bounds().width;
    int height=bounds().height;
//    int meterBorder=width/50;
    int t_base=(int)((double)height*0.7);
    int caption_y=(int)((double)height*0.85);
    int t_size=t_base-height/10;
    int t_width=t_size/10;
    int bulbDiameter=(int)((double)t_width*1.5);
    int bulbRadius=bulbDiameter/2;
    int t_xpos=(width/2)-(t_width/2);

    g.setColor(panelColor);

    g.fill3DRect(0,0,width, height, true);

    g.setColor(Color.white);
    g.fillRoundRect(t_xpos,
		    t_base-t_size-t_width,
		    t_width,
		    t_size+t_width*2,
		    t_width,
		    t_width);

    g.fillOval(t_xpos+(t_width/2)-bulbRadius,
	       t_base,
	       bulbDiameter,
	       bulbDiameter);

    g.setColor(Color.black);

    double sizeGrad=((double)t_size/(highestValue-lowestValue));

    g.setFont(numbersFont);

    for(double i=lowestValue;i<=highestValue;i=i+scaleGrad){

      int mark=t_base+(int)(lowestValue-(i*sizeGrad));
      g.drawLine(t_xpos+t_width+3, mark, t_xpos+t_width+8, mark);

      g.drawString(String.valueOf(i), t_xpos+t_width+10, mark);

    }

    g.setColor(Color.red);
    g.drawLine (t_xpos+(t_width/2),
		t_base+5,
		t_xpos+(t_width/2),
		t_base+(int)(lowestValue-(value*sizeGrad)));

    // draw caption
    g.setColor(Color.black);
    g.setFont(captionFont);
    int caption_length=g.getFontMetrics().stringWidth(caption);
    g.drawString(caption,(width/2)-(caption_length/2),caption_y);
  }

  public Dimension minimumSize() 
  { return new Dimension(250, 150); }

  public Dimension preferredSize() 
  { return minimumSize(); }
}
