(module-name 'Control)

; NOTE!!! Doesn't test for session ids in the
; parameters, see get-parameter-values.

; !!! Check all calls to eval, and quote list
; arguments if necessary to prevent evaluation.

(module-static #t)

(require <kb7.mm3.general.Strings>)
(require <kb7.kawa.general.pregexp>)
(require <kb7.wsm.PageFileToFunction>)
(require <kb7.wsm.Environments>)
(require <kb7.wsm.Log>)


; This function is called directly from the servlet's doPost
; and doGet methods. See the comments at the end of the module.
; init-document-root is the document root passed in the
; servlet initialisation parameter, and can be null. 
; The document root finally used will be the initialisation 
; parameter if this was not null, otherwise the real path for
; the URI "/". 
; init-sitedefs-name is the name of the sitedefs.scm
; file, and can be null. If it is, the sitedefs.scm file will 
; be looked for as sitedefs.scm under the document root.
;
(define (handle-get (req <javax.servlet.http.HttpServletRequest>)
                    (res <javax.servlet.http.HttpServletResponse>)
                    (init-document-root <java.lang.String>)
                    (init-sitedefs-name <java.lang.String>)
                    (init-local-sitedefs-name <java.lang.String>)
                    (parameters <java.util.Hashtable>)) <void>
(wsm-log "Entered handle-get")
;  (try-catch 
    (handle-get-1 req res init-document-root init-sitedefs-name init-local-sitedefs-name parameters)
;  ;exceptionally
;    (ex <java.lang.Throwable> (report-exception ex res))
;  )
)


(define (report-exception (ex <java.lang.Throwable>)
                          (res <javax.servlet.http.HttpServletResponse>)) <void>
(wsm-log "Caught exception") 
(wsm-log "Exception " ex)
  (invoke res 'setContentType 'text/html)
  (let ( (out <java.io.PrintWriter> (invoke res 'getWriter))
         (title (if (instance? ex <kb7.wsm.WSMException>)
                  (invoke ex 'getErrorType)
                ;else
                  "Error"
                )
         )
       )
    (invoke out 'println (string-append "<HTML><HEAD><TITLE>" title "</TITLE></HEAD>"))
    (invoke out 'println (string-append "<BODY BGCOLOR=\"white\"><H1>" title "</H1>"))
    (if (instance? ex <kb7.wsm.WSMException>)
      (begin
        (invoke out 'println "<PRE>")
        (invoke out 'println (invoke ex 'getMessage))
        (invoke ex 'printStackTrace out)
        (invoke out 'println "</PRE>")
      )
    ;else
      (begin
        (invoke out 'println "<PRE>")
        (invoke ex 'printStackTrace out)
        (invoke out 'println "</PRE>")
      )
    )
    (invoke out 'println (string-append "</BODY></HTML>"))
  )
)


; This is the guts of the function. It may throw an exception, which
; will be caught by handle-get.
; 'parameters' is supplied either by the file-upload part of 
; WSMServlet or directly from req, and is a map from name (a symbol)
; to value (a string), or if more than one value, a list of strings.
;
(define (handle-get-1 (req <javax.servlet.http.HttpServletRequest>)
                      (res <javax.servlet.http.HttpServletResponse>)
                      (init-document-root <java.lang.String>)
                      (init-sitedefs-name <java.lang.String>)
                      (init-local-sitedefs-name <java.lang.String>)
                      (parameters <java.util.Hashtable>)) <void>

(wsm-log "Entered handle-get-1") 
(wsm-log "init-document-root " init-document-root) 
(wsm-log "init-sitedefs-name " init-sitedefs-name) 
(wsm-log "init-local-sitedefs-name " init-local-sitedefs-name) 

  (let* ( (document-root <string> (get-document-root req init-document-root))
          ; What we'll take as our actual document root, i.e. location
          ; of the .pfn files for this request. Derived from the
          ; URI if init-document-root was null.

          (sitedefs-name <string> (get-sitedefs-name document-root init-sitedefs-name))
          ; Full name of the user's sitedefs file.

          (local-sitedefs-name <string> (get-local-sitedefs-name init-local-sitedefs-name))

          (raw-page-name <string> (string-append (invoke req
'getRequestURI)))
          ; Page name as given in the URI. This can be an actual page
          ; name, but could also be an action specifier, if it ends
          ; with ".action".

          (has-action <boolean> (pregexp-match ".*\\.action"
raw-page-name))
          ; True if it is an action specifier.

          (E (interaction-environment))
          ; The environment in which the page functions will operate.
        )

(wsm-log "document root " document-root) 
(wsm-log "sitedefs name " sitedefs-name) 
(wsm-log "local-sitedefs name " local-sitedefs-name) 

(wsm-log "Before loading sitedefs") 
  (load sitedefs-name)
(wsm-log "After loading sitedefs") 
  ; So that we can interrogate wsm-session? in this function.

(when (not (eq? local-sitedefs-name #!null))  
(wsm-log "Before loading local-sitedefs") 
    (load local-sitedefs-name)
(wsm-log "After loading local-sitedefs") 
)

;(wsm-log "E " (environment-bindings->string E)) 

  ;(environment-add-source-file! E sitedefs-name)
  ; For the page calls, which are all evaluated in an environment that
  ; must contain the user's functions. 
  ; I have commented the above out, since it turns out that
  ; (interaction-environment) returns the current environment
  ; anyway. Hence the call of 'load' above has already loaded
  ; the source file.

  (load-module-exports E sitedefs-name)
  ; Re-reads the sitedefs file and puts the exported names from all
  ; the required modules into E. Neither call above will go deep
  ; enough to do that.

;(wsm-log "E " (environment-bindings->string E)) 
;(wsm-log "After module exports") 

  (environment-add-module! E <kb7.wsm.PageFileToFunction>)
  (environment-add-module! E <kb7.wsm.StandardFunctions>)
  ; Get the page-function creator into E, since page functions can create other
  ; page functions. Also get the wsm standard functions.

  (environment-add-binding! E 'wsm-request req)
  (environment-add-binding! E 'wsm-response res)
  ; So that the page functions can see these as global variables.

;(wsm-log "E " (environment-bindings->string E)) 

  (let* ( (has-parameters <boolean> (not (invoke parameters 'isEmpty)))
          ; True if the request had any query parameters. Some servlet
          ; engines may store the session id as a query parameter. If so,
          ; the call to get-parameter-values above should have removed
          ; this, so that the isEmpty test still works.

          (has-session <boolean> wsm-session?)
          ; True if the user wanted sessions. 

          (result <string> #!null)
          ; Will become the output to the Web server.

          (redirected #!null)   
          ; Will become true if this request results in a redirection URL,
          ; and doesn't send any output of its own.

          (session <javax.servlet.http.HttpSession> #!null)
          ; If has-session, will become the Session instance. Irrelevant 
          ; otherwise.
        )

    (when has-session
      (set! session (invoke req 'getSession #t))
    )
    ; Create or retrieve the Session instance.

(wsm-log "has-session " has-session) 
(wsm-log "session " session) 

    ; A request can be:
    ; 1) A request for a page without any query parameters. If so,
    ;    create and evaluate the page function, calling it with
    ;    no arguments.
    ; 2) A request for a page with query parameters. If so,
    ;    create and evaluate the page function, calling it with
    ;    one argument which is the parameters map.
    ; 3) A request for a page with one query parameter REDIRECTED=true.
    ;    If so, this is the result of a previous call which has
    ;    sent a redirection and stored in the session the arguments of the 
    ;    page to which we're redirecting. Create and
    ;    evaluate the page function, calling it on them.
    ; 4) An action specifier, ending in the .action extension. If so,
    ;    it specifies a state transition. Retrieve the state from the
    ;    session, call the user's next-page function to get the new
    ;    state, page, and its arguments, then redirect to that new page.

    (if (not has-action)
      (let ( (page-name <string> (get-page-name req document-root))
           )
        (if (not has-parameters)
          (begin
(wsm-log "Case 1 - about to call " page-name) 
            (set! result (eval `(call-page ,page-name () ,document-root) E))
(wsm-log "Called the page") 
            (set! redirected #f)
          )
          ; Case 1.
        ;else
          (if (eq? (invoke parameters 'get 'REDIRECTED) #!null)
            (begin
(wsm-log "Case 2 - about to call " page-name) 
              (set! result (eval `(call-page ,page-name (list ,parameters) ,document-root) E))
              (set! redirected #f)
            )
            ; Case 2.
          ;else
            (let ( (arguments (invoke session 'getValue 'arguments))
                 )
(wsm-log "Case 3 - about to call " page-name) 
              (set! result (eval `(call-page ,page-name (quote ,arguments) ,document-root) E))
              (set! redirected #f)
            )
            ; Case 3.
          )
        )
      )
    ;else
      (let* ( (state (invoke session 'getValue 'state))
              ; Current value of the state. Can be null, if this
              ; is the starting point of a series of transitions.

(dummy1 (begin (wsm-log "Case 4")  #!null))
(dummy2 (begin (wsm-log "state " state)  #!null))

              (action <symbol> (get-action req))
              ; The action specifier, stripped of its extension
              ; (which is only there to fool the server into
              ; calling this servlet).

(dummy3 (begin (wsm-log "action " action)  #!null))

              (next-page-state-and-args <list> ((eval 'wsm-next-page E)
state action parameters))
              ; Result of the user's next-page function.
              ; I'm not sure whether wsm-next-page will be bound correctly if
              ; I make a straight reference to it here, rather than hiding
              ; as a symbol.

              (next-page <string> (car next-page-state-and-args))
              ; The next page.

(dummy45 (begin (wsm-log "next-page " next-page) #!null))
 
              (next-state (cadr next-page-state-and-args))
              ; The next state.

(dummy5 (begin (wsm-log "next-state " next-state)  #!null))

              (args <list> (caddr next-page-state-and-args))
              ; The arguments to the function that implements the
              ; next page.

              (new-url <java.lang.String> (invoke (string-append next-page "?REDIRECTED=true") 'toString))
              ; The new URL with a parameter indicating redirection to case 3.


(dummy56 (begin (wsm-log "new-url " new-url)  #!null))

              (new-encoded-url <java.lang.String> (invoke-static <kb7.wsm.Utilities> 'encodeUrl res new-url))

(dummy57 (begin (wsm-log "new-encoded-url " new-encoded-url)  #!null))

            )

(begin (wsm-log "args " args)  #!null)

        (invoke session 'putValue 'state next-state) 
        ; Store the next state in the Session.

        (invoke session 'putValue 'arguments args) 
        ; Store the page arguments. These will be retrieved by
        ; the next request, which will be case 3 above. 

        (invoke res 'sendRedirect new-encoded-url)
        (set! redirected #t)
        ; Redirect to that request.
      )
      ; Case 4.
    )

(wsm-log "Finished, ready to write if not redirected") 

    (when (not redirected)
      (invoke res 'setContentType 'text/html)

(wsm-log "About to expand SSIs") 

      (let ( (out <java.io.PrintWriter> (invoke res 'getWriter))

;             (expanded-result <java.lang.StringBuffer> (invoke-static
;<kb7.wsm.Includes> 'doIncludes 
; 
;(invoke result 'toString) 
; 
;(invoke document-root 'toString)
;                              )
;             )
           )

(wsm-log "Expanded SSIs") 

        (invoke-static <kb7.wsm.FastIncludes> 'doIncludes 
                                              (invoke result 'toString) 
                                              (invoke document-root 'toString)
                                              out
        )
        (invoke out 'flush)
      )
    )
    ; Send our own output if not redirected,
    ; after expanding any server-side includes in
    ; it.

  ))
)


; If the initialisation parameter "document-root" was set, returns
; its value, otherwise returns that for the URI "/".
; Contains file separators for the current system; may or may not end
; with one.
;
(define (get-document-root (req <javax.servlet.http.HttpServletRequest>)
                           (init-param <java.lang.String>)) <string>
  (if (not (eq? init-param #!null))
    (string-append init-param)
  ;else
    (string-append (invoke req 'getRealPath '/))
  )
)


; If the initialisation parameter "sitedefs-name" was set,
; returns its value, otherwise returns document-root.
; Contains file separators for the current system; may or may not end
; with one.
;
(define (get-sitedefs-name (document-root <string>)
                           (init-param <java.lang.String>)) <string>
  (if (eq? init-param #!null)
    (string-append (invoke-static <kb7.wsm.Utilities> 'joinDirToFile 
                                                      (invoke document-root 'toString)
                                                     'sitedefs.scm
                   )
    )
  ;else
    (string-append init-param)
  )
)


; If the initialisation parameter "init-local-sitedefs-name" was set,
; returns its value, otherwise returns null.
;
(define (get-local-sitedefs-name (init-param <java.lang.String>)) <string>
  (if (eq? init-param #!null)
    #!null
  ;else
    (string-append init-param)
  )
)


; Gets the full filename of the current page from the URI with
; which this servlet was invoked. Contains file separators for
; the current system.
;
(define (get-page-name (req <javax.servlet.http.HttpServletRequest>)
                       (document-root <string>)) <string>
  (let* ( (separator-char (static-field <java.io.File> 'separatorChar))
          (raw-name <java.lang.String> (invoke req 'getRequestURI))
          ; From a URI like /file.pfn .
          (name-with-correct-separators <java.lang.String> (invoke-static <kb7.wsm.Utilities> 'replaceFileSeparator raw-name))
        )
    (string-append (invoke-static <kb7.wsm.Utilities> 'joinDirToFile 
                                                      (invoke document-root 'toString)
                                                      name-with-correct-separators)
    )
  )
)


; Gets the action from the URI. This strips off the ".action"
; suffix, and any path parts (separated by "/") before the final
; filename.
;
(define (get-action (req <javax.servlet.http.HttpServletRequest>)) <symbol>
  (let* ( (raw-name <string> (string-append (invoke req 'getRequestURI)))
          (reversed (string-reverse raw-name))
          (without.action (drop reversed "noitca.") )    
          (contains-slash (string-contains without.action "/")) 
          (reversed-body (if contains-slash (take without.action "/") without.action))
        )
    ; We did use pregexp, but it sometimes goes into a loop, at
    ; least at ILRT.
    (string->symbol (string-reverse reversed-body))
  ) 
)


; Given the name of a sitedefs file (i.e. a source 
; file, which may have some 'require' statements in it),
; re-reads it and puts the exported names from all
; the required modules into environment.
;
(define (load-module-exports (environment <gnu.mapping.Environment>)
                             (filename <string>)) <void>
  (with-input-from-file filename
                        (lambda ()
                          (read-and-load-modules environment)
                        )
  )
)


(define (read-and-load-modules (environment <gnu.mapping.Environment>)) <void>
  (let ( (expr (read))
       )
    (when (not (eq? expr #!eof))
      (when (and (list? expr)
                 (eq? (length expr) 2)
                 (equal? (car expr) 'require)
                 (symbol? (cadr expr))
                 ; Hope it's a type name.
            )
        (environment-add-module! environment (eval (cadr expr)))
        ; The type specifier comes in as a symbol. We
        ; have to eval it to turn it into a type.
      )
      (read-and-load-modules environment)
    )
  )
)



